@@ -30,6 +30,7 @@
 
 
 #include <osgDB/FileUtils>
+#include <osgDB/fstream>
 #include <osgDB/ReadFile>
 
 #include <osgUtil/GLObjectsVisitor>
@@ -120,13 +121,13 @@ public:
     
     bool readMasterFile(Files& files) const
     {
-        std::ifstream fin(_filename.c_str());
+        osgDB::ifstream fin(_filename.c_str());
         if (fin)
         {
             osgDB::Input fr;
             fr.attach(&fin);
 
-            bool readFilename;
+            bool readFilename = false;
 
             while(!fr.eof())
             {
@@ -646,7 +647,7 @@ int main(int argc, char** argv)
 
     std::string filterName;
 
-    osgTerrain::Layer::Filter filter = osgTerrain::Layer::LINEAR;
+    osg::Texture::FilterMode filter = osg::Texture::LINEAR;
 
     bool readParameter = false;
     float minValue, maxValue;
@@ -705,7 +706,7 @@ int main(int argc, char** argv)
                 
                 hfl->setLocator(locator.get());
                 hfl->setValidDataOperator(validDataOperator.get());
-                hfl->setFilter(filter);
+                hfl->setMagFilter(filter);
                 
                 if (offset!=0.0f || scale!=1.0f)
                 {
@@ -740,7 +741,7 @@ int main(int argc, char** argv)
                 imageLayer->setImage(image.get());
                 imageLayer->setLocator(locator.get());
                 imageLayer->setValidDataOperator(validDataOperator.get());
-                imageLayer->setFilter(filter);
+                imageLayer->setMagFilter(filter);
                 
                 if (offset!=0.0f || scale!=1.0f)
                 {
@@ -775,7 +776,7 @@ int main(int argc, char** argv)
                 imageLayer->setImage(image.get());
                 imageLayer->setLocator(locator.get());
                 imageLayer->setValidDataOperator(validDataOperator.get());
-                imageLayer->setFilter(filter);
+                imageLayer->setMagFilter(filter);
                 
                 if (offset!=0.0f || scale!=1.0f)
                 {
@@ -805,11 +806,11 @@ int main(int argc, char** argv)
             if (filterName=="NEAREST")
             {
                 osg::notify(osg::NOTICE)<<"--filter "<<filterName<<std::endl;
-                filter = osgTerrain::Layer::NEAREST;
+                filter = osg::Texture::NEAREST;
             }
             else if (filterName=="LINEAR") 
             {
-                filter = osgTerrain::Layer::LINEAR;
+                filter = osg::Texture::LINEAR;
                 osg::notify(osg::NOTICE)<<"--filter "<<filterName<<std::endl;
             }
             else
@@ -819,7 +820,7 @@ int main(int argc, char** argv)
 
             if (terrainTile->getColorLayer(layerNum))
             {
-                terrainTile->getColorLayer(layerNum)->setFilter(filter);
+                terrainTile->getColorLayer(layerNum)->setMagFilter(filter);
             }
             
         }
@@ -830,15 +831,17 @@ int main(int argc, char** argv)
 
             osg::ref_ptr<osg::TransferFunction1D> tf = new osg::TransferFunction1D;
             
-            tf->setInputRange(minValue, maxValue);
+            unsigned int numCells = 6;
+            float delta = (maxValue-minValue)/float(numCells-1);
+            float v = minValue;
             
             tf->allocate(6);
-            tf->setValue(0, osg::Vec4(1.0,1.0,1.0,1.0));
-            tf->setValue(1, osg::Vec4(1.0,0.0,1.0,1.0));
-            tf->setValue(2, osg::Vec4(1.0,0.0,0.0,1.0));
-            tf->setValue(3, osg::Vec4(1.0,1.0,0.0,1.0));
-            tf->setValue(4, osg::Vec4(0.0,1.0,1.0,1.0));
-            tf->setValue(5, osg::Vec4(0.0,1.0,0.0,1.0));
+            tf->setColor(v, osg::Vec4(1.0,1.0,1.0,1.0)); v += delta;
+            tf->setColor(v, osg::Vec4(1.0,0.0,1.0,1.0)); v += delta;
+            tf->setColor(v, osg::Vec4(1.0,0.0,0.0,1.0)); v += delta;
+            tf->setColor(v, osg::Vec4(1.0,1.0,0.0,1.0)); v += delta;
+            tf->setColor(v, osg::Vec4(0.0,1.0,1.0,1.0)); v += delta;
+            tf->setColor(v, osg::Vec4(0.0,1.0,0.0,1.0));
             
             osg::notify(osg::NOTICE)<<"--tf "<<minValue<<" "<<maxValue<<std::endl;
 
