@@ -19,6 +19,7 @@
 #include <string>
 #include <iostream>
 #include <process.h>
+#include <stdlib.h>
 
 #if defined(_MSC_VER) && (_MSC_VER < 1300)
 #ifdef __SGI_STL
@@ -97,7 +98,7 @@ namespace OpenThreads {
 
             if (thread->_prvData==0) return 0;
 
-            TlsSetValue(Win32ThreadPrivateData::TLS.ID ,data);
+            TlsSetValue(Win32ThreadPrivateData::TLS.getId(), data);
             //---------------------------------------------------------------------
             // Set the proper scheduling priorities
             //
@@ -108,6 +109,9 @@ namespace OpenThreads {
             // release the thread that created this thread.
             pd->threadStartedBlock.release();
 
+            if (0 <= pd->cpunum)
+                thread->setProcessorAffinity(pd->cpunum);
+
             try{
                 thread->run();
             }
@@ -123,6 +127,7 @@ namespace OpenThreads {
                 // abnormal termination but must be caught in win32 anyway
             }
 
+            TlsSetValue(Win32ThreadPrivateData::TLS.getId(), 0);
             pd->isRunning = false;
 
             return 0;
@@ -132,8 +137,8 @@ namespace OpenThreads {
         // Print information related to thread schduling parameters.
         //
         static void PrintThreadSchedulingInfo(Thread *thread) {
-            Win32ThreadPrivateData *pd =
-                static_cast<Win32ThreadPrivateData *>(thread->_prvData);
+
+            // Win32ThreadPrivateData *pd = static_cast<Win32ThreadPrivateData *>(thread->_prvData);
 
             std::cout<<"Thread "<< thread <<" priority : ";
 
@@ -203,7 +208,7 @@ namespace OpenThreads {
 
 Thread* Thread::CurrentThread()
 {
-    return (Thread* )TlsGetValue(Win32ThreadPrivateData::TLS.ID);
+    return (Thread* )TlsGetValue(Win32ThreadPrivateData::TLS.getId());
 };
 
 //----------------------------------------------------------------------------
@@ -247,6 +252,7 @@ Thread::Thread() {
     pd->threadPolicy = Thread::THREAD_SCHEDULE_DEFAULT;
     pd->detached = false;
     pd->cancelEvent.set(CreateEvent(NULL,TRUE,FALSE,NULL));
+    pd->cpunum = -1;
 
     _prvData = static_cast<void *>(pd);
 }
@@ -339,13 +345,13 @@ int Thread::start() {
 
     pd->uniqueId = (int)ID;
 
-    // wait till the thread has actually started.
-    pd->threadStartedBlock.block();
-
     if(!pd->tid) {
         return -1;
     }
 
+    // wait till the thread has actually started.
+    pd->threadStartedBlock.block();
+
     return 0;
 
 }
@@ -436,8 +442,7 @@ int Thread::testCancel()
 //    ExitThread(0);
     throw Win32ThreadCanceled();
 
-    return 0;
-
+//    return 0;
 }
 
 
@@ -567,6 +572,14 @@ size_t Thread::getStackSize() {
 int Thread::setProcessorAffinity( unsigned int cpunum )
 {
     Win32ThreadPrivateData *pd = static_cast<Win32ThreadPrivateData *> (_prvData);
+    pd->cpunum = cpunum;
+    if (!pd->isRunning)
+       return 0;
+
+    if (pd->tid.get() == INVALID_HANDLE_VALUE)
+       return -1;
+
+
     DWORD affinityMask  = 0x1 << cpunum ; // thread affinity mask
     DWORD_PTR res =
         SetThreadAffinityMask
