@@ -13,6 +13,7 @@
 #include <osg/GLExtensions>
 #include <osg/Texture3D>
 #include <osg/State>
+#include <osg/ImageSequence>
 #include <osg/GLU>
 #include <osg/Notify>
 
@@ -30,6 +31,16 @@ Texture3D::Texture3D():
 {
 }
 
+
+Texture3D::Texture3D(Image* image):
+            _textureWidth(0),
+            _textureHeight(0),
+            _textureDepth(0),
+            _numMipmapLevels(0)
+{
+    setImage(image);
+}
+
 Texture3D::Texture3D(const Texture3D& text,const CopyOp& copyop):
             Texture(text,copyop),
             _image(copyop(text._image.get())),
@@ -96,12 +107,26 @@ int Texture3D::compare(const StateAttribute& sa) const
 
 void Texture3D::setImage(Image* image)
 {
+    if (_image == image) return;
+
+    if (dynamic_cast<osg::ImageSequence*>(_image.get()))
+    {
+        setUpdateCallback(0);
+        setDataVariance(osg::Object::STATIC);
+    }
+
     // delete old texture objects.
     dirtyTextureObject();
 
     _modifiedCount.setAllElementsTo(0);
 
     _image = image;
+    
+    if (dynamic_cast<osg::ImageSequence*>(_image.get()))
+    {
+        setUpdateCallback(new ImageSequence::UpdateCallback());
+        setDataVariance(osg::Object::DYNAMIC);
+    }
 }
 
 void Texture3D::computeRequiredTextureDimensions(State& state, const osg::Image& image,GLsizei& inwidth, GLsizei& inheight,GLsizei& indepth, GLsizei& numMipmapLevels) const
@@ -134,7 +159,7 @@ void Texture3D::computeRequiredTextureDimensions(State& state, const osg::Image&
     inheight = height;
     indepth = depth;
     
-    bool useHardwareMipMapGeneration = false; //!image.isMipmap() && _useHardwareMipMapGeneration && extensions->isGenerateMipMapSupported();
+    bool useHardwareMipMapGeneration = !image.isMipmap() && _useHardwareMipMapGeneration && texExtensions->isGenerateMipMapSupported();
 
     if( _min_filter == LINEAR || _min_filter == NEAREST || useHardwareMipMapGeneration )
     {
@@ -195,6 +220,8 @@ void Texture3D::apply(State& state) const
         }
         else if (_image.get() && getModifiedCount(contextID) != _image->getModifiedCount())
         {
+           computeRequiredTextureDimensions(state,*_image,_textureWidth, _textureHeight, _textureDepth,_numMipmapLevels);
+
             applyTexImage3D(GL_TEXTURE_3D,_image.get(),state, _textureWidth, _textureHeight, _textureDepth,_numMipmapLevels);
 
             // update the modified count to show that it is upto date.
@@ -231,7 +258,7 @@ void Texture3D::apply(State& state) const
         // compute the dimensions of the texture.
         computeRequiredTextureDimensions(state,*_image,_textureWidth, _textureHeight, _textureDepth,_numMipmapLevels);
 
-        _textureObjectBuffer[contextID] = textureObject = generateTextureObject(contextID,GL_TEXTURE_3D);
+        textureObject = generateTextureObject(contextID,GL_TEXTURE_3D);
 
         textureObject->bind();
 
@@ -245,6 +272,8 @@ void Texture3D::apply(State& state) const
         // update the modified count to show that it is upto date.
         getModifiedCount(contextID) = _image->getModifiedCount();
 
+        _textureObjectBuffer[contextID] = textureObject;
+
         if (_unrefImageDataAfterApply && areAllTextureObjectsLoaded() && _image->getDataVariance()==STATIC)
         {
             Texture3D* non_const_this = const_cast<Texture3D*>(this);
@@ -327,13 +356,21 @@ void Texture3D::applyTexImage3D(GLenum target, Image* image, State& state, GLsiz
 
     glPixelStorei(GL_UNPACK_ALIGNMENT,image->getPacking());
 
-    if( _min_filter == LINEAR || _min_filter == NEAREST )
+    bool useHardwareMipMapGeneration = !image->isMipmap() && _useHardwareMipMapGeneration && texExtensions->isGenerateMipMapSupported();
+
+    if( _min_filter == LINEAR || _min_filter == NEAREST || useHardwareMipMapGeneration )
     {
+        bool hardwareMipMapOn = false;
+        if (_min_filter != LINEAR && _min_filter != NEAREST) 
+        {
+            if (useHardwareMipMapGeneration) glTexParameteri(GL_TEXTURE_3D, GL_GENERATE_MIPMAP_SGIS,GL_TRUE);
+            hardwareMipMapOn = true;
+        }
+
         numMipmapLevels = 1;
 
         if (!compressed_image)
         {
-            // notify(WARN)<<"glTexImage3D"<<std::endl;
             extensions->glTexImage3D( target, 0, _internalFormat,
                                       inwidth, inheight, indepth,
                                       _borderWidth,
@@ -356,7 +393,7 @@ void Texture3D::applyTexImage3D(GLenum target, Image* image, State& state, GLsiz
                 image->data());
         }
 
-
+        if (hardwareMipMapOn) glTexParameteri(GL_TEXTURE_3D, GL_GENERATE_MIPMAP_SGIS,GL_FALSE);
     }
     else
     {
@@ -447,7 +484,6 @@ void Texture3D::allocateMipmap(State& state) const
     if (textureObject && _textureWidth != 0 && _textureHeight != 0 && _textureDepth != 0)
     {
         const Extensions* extensions = getExtensions(contextID,true);
-        const Texture::Extensions* texExtensions = Texture::getExtensions(contextID,true);
     
         // bind texture
         textureObject->bind();
@@ -546,12 +582,12 @@ void Texture3D::Extensions::setupGLExtensions(unsigned int contextID)
     
     glGetIntegerv(GL_MAX_3D_TEXTURE_SIZE, &_maxTexture3DSize);
 
-    _glTexImage3D                     = getGLExtensionFuncPtr("glTexImage3D","glTexImage3DEXT");
-    _glTexSubImage3D                  = getGLExtensionFuncPtr("glTexSubImage3D","glTexSubImage3DEXT");
-    _glCompressedTexImage3D           = getGLExtensionFuncPtr("glCompressedTexImage3D","glCompressedTexImage3DARB");
-    _glCompressedTexSubImage3D        = getGLExtensionFuncPtr("glCompressedTexSubImage3D","glCompressedTexSubImage3DARB");
-    _glCopyTexSubImage3D              = getGLExtensionFuncPtr("glCopyTexSubImage3D","glCopyTexSubImage3DEXT");
-    _gluBuild3DMipmaps                = getGLExtensionFuncPtr("gluBuild3DMipmaps");
+    setGLExtensionFuncPtr(_glTexImage3D,"glTexImage3D","glTexImage3DEXT");
+    setGLExtensionFuncPtr(_glTexSubImage3D,"glTexSubImage3D","glTexSubImage3DEXT");
+    setGLExtensionFuncPtr(_glCompressedTexImage3D,"glCompressedTexImage3D","glCompressedTexImage3DARB");
+    setGLExtensionFuncPtr(_glCompressedTexSubImage3D,"glCompressedTexSubImage3D","glCompressedTexSubImage3DARB");
+    setGLExtensionFuncPtr(_glCopyTexSubImage3D,"glCopyTexSubImage3D","glCopyTexSubImage3DEXT");
+    setGLExtensionFuncPtr(_gluBuild3DMipmaps,"gluBuild3DMipmaps");
 
 }
 
@@ -560,8 +596,7 @@ void Texture3D::Extensions::glTexImage3D( GLenum target, GLint level, GLenum int
 //    ::glTexImage3D( target, level, internalFormat, width, height, depth, border, format, type, pixels);
     if (_glTexImage3D)
     {
-        typedef void (APIENTRY * GLTexImage3DProc)      ( GLenum target, GLint level, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, const GLvoid *pixels);
-        ((GLTexImage3DProc)_glTexImage3D)( target, level, internalFormat, width, height, depth, border, format, type, pixels);
+        _glTexImage3D( target, level, internalFormat, width, height, depth, border, format, type, pixels);
     }
     else
     {
@@ -574,8 +609,7 @@ void Texture3D::Extensions::glTexSubImage3D( GLenum target, GLint level, GLint x
 //    ::glTexSubImage3D( target, level, xoffset, yoffset, zoffset, width, height, depth, format, type, pixels);
     if (_glTexSubImage3D)
     {
-        typedef void (APIENTRY * GLTexSubImage3DProc)   ( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, const GLvoid *pixels);
-        ((GLTexSubImage3DProc)_glTexSubImage3D)( target, level, xoffset, yoffset, zoffset, width, height, depth, format, type, pixels);
+        _glTexSubImage3D( target, level, xoffset, yoffset, zoffset, width, height, depth, format, type, pixels);
     }
     else
     {
@@ -587,8 +621,7 @@ void Texture3D::Extensions::glCompressedTexImage3D(GLenum target, GLint level, G
 {
     if (_glCompressedTexImage3D)
     {
-        typedef void (APIENTRY * CompressedTexImage3DArbProc) (GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, const GLvoid *data);
-        ((CompressedTexImage3DArbProc)_glCompressedTexImage3D)(target, level, internalformat, width, height, depth, border, imageSize, data);
+        _glCompressedTexImage3D(target, level, internalformat, width, height, depth, border, imageSize, data);
     }
     else
     {
@@ -600,8 +633,7 @@ void Texture3D::Extensions::glCompressedTexSubImage3D( GLenum target, GLint leve
 {
     if (_glCompressedTexSubImage3D)
     {
-        typedef void (APIENTRY * CompressedTexSubImage3DArbProc) (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, const GLvoid *data);
-        ((CompressedTexSubImage3DArbProc)_glCompressedTexSubImage3D)(target, level, xoffset, yoffset, zoffset, width, height, depth, format, imageSize, data);
+        _glCompressedTexSubImage3D(target, level, xoffset, yoffset, zoffset, width, height, depth, format, imageSize, data);
     }
     else
     {
@@ -614,8 +646,7 @@ void Texture3D::Extensions::glCopyTexSubImage3D( GLenum target, GLint level, GLi
 //    ::glCopyTexSubImage3D(target, level, xoffset, yoffset, zoffset, x, y, width, height);
     if (_glCopyTexSubImage3D)
     {
-        typedef void (APIENTRY * GLCopyTexSubImageProc) ( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height );
-        ((GLCopyTexSubImageProc)_glCopyTexSubImage3D)(target, level, xoffset, yoffset, zoffset, x, y, width, height);
+        _glCopyTexSubImage3D(target, level, xoffset, yoffset, zoffset, x, y, width, height);
     }
     else
     {
@@ -628,8 +659,7 @@ void Texture3D::Extensions::gluBuild3DMipmaps( GLenum target, GLint internalForm
 //    ::gluBuild3DMipmaps(target, internalFormat, width, height, depth, format, type, data);
     if (_gluBuild3DMipmaps)
     {
-        typedef void (APIENTRY * GLUBuild3DMipMapsProc) ( GLenum target, GLint internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, const GLvoid *data);
-        ((GLUBuild3DMipMapsProc)_gluBuild3DMipmaps)(target, internalFormat, width, height, depth, format, type, data);
+        _gluBuild3DMipmaps(target, internalFormat, width, height, depth, format, type, data);
     }
     else
     {
