@@ -23,15 +23,19 @@
 #include <osg/Geode>
 #include <osg/ApplicationUsage>
 #include <osg/Version>
+#include <osg/Timer>
 
 #include <osgDB/Registry>
 #include <osgDB/FileUtils>
 #include <osgDB/FileNameUtils>
+#include <osgDB/fstream>
 #include <osgDB/Archive>
 
 #include <algorithm>
 #include <set>
 
+#include <stdlib.h>
+
 #if defined(__sgi)
     #include <ctype.h>
 #elif defined(__GNUC__) || !defined(WIN32) || defined(__MWERKS__)
@@ -39,6 +43,11 @@
     using std::tolower;
 #endif
 
+#ifdef OSG_DEBUG_POSTFIX
+    #define OSG_DEBUG_POSTFIX_WITH_QUOTES ADDQUOTES(OSG_DEBUG_POSTFIX)
+#else
+    #define OSG_DEBUG_POSTFIX_WITH_QUOTES "d"
+#endif
 
 using namespace osg;
 using namespace osgDB;
@@ -51,12 +60,15 @@ static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENT
 static osg::ApplicationUsageProxy Registry_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_LIBRARY_PATH <path>[;path]..","Paths for locating libraries/ plugins");
 #endif
 
+static osg::ApplicationUsageProxy Registry_e2(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_BUILD_KDTREES on/off","Enable/disable the automatic building of KdTrees for each loaded Geometry.");
+
 
 class Registry::AvailableReaderWriterIterator
 {
 public:
-    AvailableReaderWriterIterator(Registry::ReaderWriterList& rwList):
-        _rwList(rwList) {}
+    AvailableReaderWriterIterator(Registry::ReaderWriterList& rwList, OpenThreads::ReentrantMutex& pluginMutex):
+        _rwList(rwList),
+        _pluginMutex(pluginMutex) {}
 
 
     ReaderWriter& operator * () { return *get(); }
@@ -72,11 +84,16 @@ public:
 
 protected:
 
+    AvailableReaderWriterIterator& operator = (const AvailableReaderWriterIterator&) { return *this; }
+
     Registry::ReaderWriterList&     _rwList;
+    OpenThreads::ReentrantMutex&    _pluginMutex;
+    
     std::set<ReaderWriter*>         _rwUsed;
 
     ReaderWriter* get() 
     {
+        OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_pluginMutex);
         Registry::ReaderWriterList::iterator itr=_rwList.begin();
         for(;itr!=_rwList.end();++itr)
         {
@@ -155,6 +172,23 @@ Registry::Registry()
     // comment out because it was causing problems under OSX - causing it to crash osgconv when constructing ostream in osg::notify().
     // notify(INFO) << "Constructing osg::Registry"<<std::endl;
 
+    _buildKdTreesHint = ReaderWriter::Options::NO_PREFERENCE;
+    _kdTreeBuilder = new osg::KdTreeBuilder;
+    
+    const char* kdtree_str = getenv("OSG_BUILD_KDTREES");
+    if (kdtree_str)
+    {
+        bool switchOff = (strcmp(kdtree_str, "off")==0 || strcmp(kdtree_str, "OFF")==0 || strcmp(kdtree_str, "Off")==0 );
+        if (switchOff) _buildKdTreesHint = ReaderWriter::Options::DO_NOT_BUILD_KDTREES;
+        else _buildKdTreesHint = ReaderWriter::Options::BUILD_KDTREES;
+    }
+
+    const char* fileCachePath = getenv("OSG_FILE_CACHE");
+    if (fileCachePath)
+    {
+        _fileCache = new FileCache(fileCachePath);    
+    }
+
     _createNodeFromImage = false;
     _openingLibrary = false;
 
@@ -163,6 +197,8 @@ Registry::Registry()
     
     initFilePathLists();
 
+
+
     // register file extension alias.
     const char* flt_str = getenv("OSG_OPEN_FLIGHT_PLUGIN");
     if (flt_str)
@@ -193,6 +229,16 @@ Registry::Registry()
     addFileExtensionAlias("ivz",   "gz");
     addFileExtensionAlias("ozg",   "gz");
     
+    addFileExtensionAlias("mag",   "dicom");
+    addFileExtensionAlias("ph",   "dicom");
+    addFileExtensionAlias("ima",   "dicom");
+    addFileExtensionAlias("dcm",   "dicom");
+    addFileExtensionAlias("dic",   "dicom");
+
+    addFileExtensionAlias("gl",   "glsl");
+    addFileExtensionAlias("vert",   "glsl");
+    addFileExtensionAlias("frag",   "glsl");
+
 #if defined(DARWIN_QUICKTIME)
     addFileExtensionAlias("jpg",  "qt");
     addFileExtensionAlias("jpe",  "qt");
@@ -211,6 +257,7 @@ Registry::Registry()
     addFileExtensionAlias("dv",   "qt");
     addFileExtensionAlias("mp4",  "qt");
     addFileExtensionAlias("m4v",  "qt");
+    addFileExtensionAlias("3gp",  "qt");
     // Add QuickTime live support for OSX
     addFileExtensionAlias("live", "qt");
 #else
@@ -222,6 +269,7 @@ Registry::Registry()
     #if defined(USE_XINE)
         addFileExtensionAlias("mov",  "xine");
         addFileExtensionAlias("mpg",  "xine");
+        addFileExtensionAlias("ogv",  "xine");
         addFileExtensionAlias("mpv",  "xine");
         addFileExtensionAlias("dv",   "xine");
         addFileExtensionAlias("avi",  "xine");
@@ -233,6 +281,8 @@ Registry::Registry()
     #if defined(USE_QUICKTIME)
         addFileExtensionAlias("mov", "qt");
         addFileExtensionAlias("live", "qt");
+        addFileExtensionAlias("mpg", "qt");
+        addFileExtensionAlias("avi", "qt");
     #endif
 #endif
 
@@ -277,14 +327,16 @@ Registry::~Registry()
 
 void Registry::destruct()
 {
-    // switch off the pager and its associated thread before we clean up 
-    // rest of the Registry.
-    _databasePager = 0;
+    // osg::notify(osg::NOTICE)<<"Registry::destruct()"<<std::endl;
 
     // clean up the SharedStateManager 
     _sharedStateManager = 0;
     
 
+    // clean up the FileCache
+    _fileCache = 0;
+    
+
     // object cache clear needed here to prevent crash in unref() of
     // the objects it contains when running the TXP plugin.
     // Not sure why, but perhaps there is is something in a TXP plugin
@@ -381,7 +433,7 @@ void Registry::readCommandLine(osg::ArgumentParser& arguments)
 
     while(arguments.read("-O",value))
     {
-        setOptions(new osgDB::ReaderWriter::Options(value));
+        setOptions(new ReaderWriter::Options(value));
     }
 }
 
@@ -489,6 +541,8 @@ void Registry::addReaderWriter(ReaderWriter* rw)
 
     // notify(INFO) << "osg::Registry::addReaderWriter("<<rw->className()<<")"<< std::endl;
 
+    OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_pluginMutex);
+
     _rwList.push_back(rw);
 
 }
@@ -500,6 +554,8 @@ void Registry::removeReaderWriter(ReaderWriter* rw)
 
 //    notify(INFO) << "osg::Registry::removeReaderWriter();"<< std::endl;
 
+    OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_pluginMutex);
+
     ReaderWriterList::iterator rwitr = std::find(_rwList.begin(),_rwList.end(),rw);
     if (rwitr!=_rwList.end())
     {
@@ -523,7 +579,7 @@ bool Registry::readPluginAliasConfigurationFile( const std::string& file )
         return false;
     }
 
-    std::ifstream ifs;
+    osgDB::ifstream ifs;
     ifs.open( fileName.c_str() );
     if (!ifs.good())
     {
@@ -584,24 +640,23 @@ std::string Registry::createLibraryNameForExtension(const std::string& ext)
     ExtensionAliasMap::iterator itr=_extAliasMap.find(lowercase_ext);
     if (itr!=_extAliasMap.end() && ext != itr->second) return createLibraryNameForExtension(itr->second);
 
-#ifdef OSG_JAVA_BUILD
+#if defined(OSG_JAVA_BUILD)
     static std::string prepend = std::string("osgPlugins-")+std::string(osgGetVersion())+std::string("/java");
-#elseif defined(__APPLE__)
-    // OSX is rather a mess in FileUtils.cpp w.r.t its appendPlatformSpecificLibraryFilePaths implementation
-    // as it hardwires the plugin name to PlugIns.  This *needs* fixing to use the naming convention as all
-    // other platforms.
-    static std::string prepend = "";
 #else
     static std::string prepend = std::string("osgPlugins-")+std::string(osgGetVersion())+std::string("/");
 #endif
 
 #if defined(__CYGWIN__)
-    return prepend+"cygwin_"+"osgdb_"+lowercase_ext+".dll";
+    #ifdef _DEBUG
+        return prepend+"cygwin_"+"osgdb_"+lowercase_ext+OSG_DEBUG_POSTFIX_WITH_QUOTES+".dll";
+    #else
+        return prepend+"cygwin_"+"osgdb_"+lowercase_ext+".dll";
+    #endif
 #elif defined(__MINGW32__)
     return prepend+"mingw_"+"osgdb_"+lowercase_ext+".dll";
 #elif defined(WIN32)
     #ifdef _DEBUG
-        return prepend+"osgdb_"+lowercase_ext+"d.dll";
+        return prepend+"osgdb_"+lowercase_ext+ OSG_DEBUG_POSTFIX_WITH_QUOTES +".dll";
     #else
         return prepend+"osgdb_"+lowercase_ext+".dll";
     #endif
@@ -611,7 +666,11 @@ std::string Registry::createLibraryNameForExtension(const std::string& ext)
     // why don't we use PLUGIN_EXT from the makefiles here?
     return prepend+"osgdb_"+lowercase_ext+".sl";
 #else
-    return prepend+"osgdb_"+lowercase_ext+".so";
+    #ifdef _DEBUG
+         return prepend+"osgdb_"+lowercase_ext+ OSG_DEBUG_POSTFIX_WITH_QUOTES + ".so";
+    #else
+         return prepend+"osgdb_"+lowercase_ext+".so";
+    #endif
 #endif
 
 }
@@ -624,7 +683,7 @@ std::string Registry::createLibraryNameForNodeKit(const std::string& name)
     return "lib"+name+".dll";
 #elif defined(WIN32)
     #ifdef _DEBUG
-        return name+"d.dll";
+        return name+OSG_DEBUG_POSTFIX_WITH_QUOTES +".dll";
     #else
         return name+".dll";
     #endif
@@ -634,31 +693,38 @@ std::string Registry::createLibraryNameForNodeKit(const std::string& name)
     // why don't we use PLUGIN_EXT from the makefiles here?
     return "lib"+name+".sl";
 #else
-    return "lib"+name+".so";
+    #ifdef _DEBUG
+        return "lib"+name+OSG_DEBUG_POSTFIX_WITH_QUOTES +".so";
+    #else
+        return "lib"+name+".so";
+    #endif
 #endif
 }
 
-bool Registry::loadLibrary(const std::string& fileName)
+Registry::LoadStatus Registry::loadLibrary(const std::string& fileName)
 {
-    DynamicLibrary* dl = getLibrary(fileName);
-    if (dl) return false;
+    OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_pluginMutex);
+
+    DynamicLibraryList::iterator ditr = getLibraryItr(fileName);
+    if (ditr!=_dlList.end()) return PREVIOUSLY_LOADED;
 
     _openingLibrary=true;
 
-    dl = DynamicLibrary::loadLibrary(fileName);
+    DynamicLibrary* dl = DynamicLibrary::loadLibrary(fileName);
     _openingLibrary=false;
 
     if (dl)
     {
         _dlList.push_back(dl);
-        return true;
+        return LOADED;
     }
-    return false;
+    return NOT_LOADED;
 }
 
 
 bool Registry::closeLibrary(const std::string& fileName)
 {
+    OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_pluginMutex);
     DynamicLibraryList::iterator ditr = getLibraryItr(fileName);
     if (ditr!=_dlList.end())
     {
@@ -670,6 +736,8 @@ bool Registry::closeLibrary(const std::string& fileName)
 
 void Registry::closeAllLibraries()
 {
+    // osg::notify(osg::NOTICE)<<"Registry::closeAllLibraries()"<<std::endl;
+    OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_pluginMutex);
     _dlList.clear();
 }
 
@@ -685,6 +753,7 @@ Registry::DynamicLibraryList::iterator Registry::getLibraryItr(const std::string
 
 DynamicLibrary* Registry::getLibrary(const std::string& fileName)
 {
+    OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_pluginMutex);
     DynamicLibraryList::iterator ditr = getLibraryItr(fileName);
     if (ditr!=_dlList.end()) return ditr->get();
     else return NULL;
@@ -695,6 +764,8 @@ ReaderWriter* Registry::getReaderWriterForExtension(const std::string& ext)
     // record the existing reader writer.
     std::set<ReaderWriter*> rwOriginal;
 
+    OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_pluginMutex);
+
     // first attemt one of the installed loaders
     for(ReaderWriterList::iterator itr=_rwList.begin();
         itr!=_rwList.end();
@@ -703,18 +774,20 @@ ReaderWriter* Registry::getReaderWriterForExtension(const std::string& ext)
         rwOriginal.insert(itr->get());
         if((*itr)->acceptsExtension(ext)) return (*itr).get();
     }
-
+    
     // now look for a plug-in to load the file.
     std::string libraryName = createLibraryNameForExtension(ext);
     notify(INFO) << "Now checking for plug-in "<<libraryName<< std::endl;
-    if (loadLibrary(libraryName))
+    if (loadLibrary(libraryName)==LOADED)
     {
         for(ReaderWriterList::iterator itr=_rwList.begin();
             itr!=_rwList.end();
             ++itr)
         {
             if (rwOriginal.find(itr->get())==rwOriginal.end())
-          if((*itr)->acceptsExtension(ext)) return (*itr).get();
+            {
+                if((*itr)->acceptsExtension(ext)) return (*itr).get();
+            }
         }
     }
 
@@ -770,21 +843,19 @@ osg::Object* Registry::readObjectOfType(const basic_type_wrapper &btw,Input& fr)
             // we have a composite name so now strip off the library name
             // are try to load it, and then retry the readObject to see
             // if we can recognize the objects.
-        
             std::string libraryName = std::string(token,0,posDoubleColon);
 
             // first try the standard nodekit library.
             std::string nodeKitLibraryName = createLibraryNameForNodeKit(libraryName);
-            if (loadLibrary(nodeKitLibraryName)) return readObjectOfType(btw,fr);
+            if (loadLibrary(nodeKitLibraryName)==LOADED) return readObjectOfType(btw,fr);
             
             // otherwise try the osgdb_ plugin library.
             std::string pluginLibraryName = createLibraryNameForExtension(libraryName);
-            if (loadLibrary(pluginLibraryName)) return readObjectOfType(btw,fr);
+            if (loadLibrary(pluginLibraryName)==LOADED) return readObjectOfType(btw,fr);
         }
     }
     else if (fr[1].isOpenBracket())
     {
-    
         DotOsgWrapper* wrapper = itr->second.get();
         const osg::Object* proto = wrapper->getPrototype();
         if (proto==NULL)
@@ -831,30 +902,26 @@ osg::Object* Registry::readObjectOfType(const basic_type_wrapper &btw,Input& fr)
                     std::string::size_type posDoubleColon = token.rfind("::");
                     if (posDoubleColon != std::string::npos)
                     {
-
                         // we have a composite name so now strip off the library name
-                        // are try to load it, and then retry the find to see
+                        // and try to load it, and then retry the find to see
                         // if we can recognize the objects.
-
                         std::string libraryName = std::string(token,0,posDoubleColon);
 
                         // first try the standard nodekit library.
                         std::string nodeKitLibraryName = createLibraryNameForNodeKit(libraryName);
-                        if (loadLibrary(nodeKitLibraryName)) 
+                        if (loadLibrary(nodeKitLibraryName)==LOADED)
                         {
                             mitr = _objectWrapperMap.find(*aitr);
-                        }
-
-                        if (mitr==_objectWrapperMap.end())
-                        {
-                            // otherwise try the osgdb_ plugin library.
-                            std::string pluginLibraryName = createLibraryNameForExtension(libraryName);
-                            if (loadLibrary(pluginLibraryName))
+                            if (mitr==_objectWrapperMap.end())
                             {
-                                mitr = _objectWrapperMap.find(*aitr);
+                                // otherwise try the osgdb_ plugin library.
+                                std::string pluginLibraryName = createLibraryNameForExtension(libraryName);
+                                if (loadLibrary(pluginLibraryName)==LOADED)
+                                {
+                                    mitr = _objectWrapperMap.find(*aitr);
+                                }
                             }
                         }
-
                     }
                 }
 
@@ -902,11 +969,11 @@ osg::Object* Registry::readObject(DotOsgWrapperMap& dowMap,Input& fr)
 
             // first try the standard nodekit library.
             std::string nodeKitLibraryName = createLibraryNameForNodeKit(libraryName);
-            if (loadLibrary(nodeKitLibraryName)) return readObject(dowMap,fr);
+            if (loadLibrary(nodeKitLibraryName)==LOADED) return readObject(dowMap,fr);
             
             // otherwise try the osgdb_ plugin library.
             std::string pluginLibraryName = createLibraryNameForExtension(libraryName);
-            if (loadLibrary(pluginLibraryName)) return readObject(dowMap,fr);
+            if (loadLibrary(pluginLibraryName)==LOADED) return readObject(dowMap,fr);
         }
     }
     else if (fr[1].isOpenBracket())
@@ -962,7 +1029,7 @@ osg::Object* Registry::readObject(DotOsgWrapperMap& dowMap,Input& fr)
 
                         // first try the standard nodekit library.
                         std::string nodeKitLibraryName = createLibraryNameForNodeKit(libraryName);
-                        if (loadLibrary(nodeKitLibraryName)) 
+                        if (loadLibrary(nodeKitLibraryName)==LOADED) 
                         {
                             mitr = _objectWrapperMap.find(*aitr);
                         }
@@ -971,7 +1038,7 @@ osg::Object* Registry::readObject(DotOsgWrapperMap& dowMap,Input& fr)
                         {
                             // otherwise try the osgdb_ plugin library.
                             std::string pluginLibraryName = createLibraryNameForExtension(libraryName);
-                            if (loadLibrary(pluginLibraryName))
+                            if (loadLibrary(pluginLibraryName)==LOADED)
                             {
                                 mitr = _objectWrapperMap.find(*aitr);
                             }
@@ -1191,11 +1258,11 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
     {
         // first try the standard nodekit library.
         std::string nodeKitLibraryName = createLibraryNameForNodeKit(libraryName);
-        if (loadLibrary(nodeKitLibraryName)) return writeObject(obj,fw);
+        if (loadLibrary(nodeKitLibraryName)==LOADED) return writeObject(obj,fw);
 
         // otherwise try the osgdb_ plugin library.
         std::string pluginLibraryName = createLibraryNameForExtension(libraryName);
-        if (loadLibrary(pluginLibraryName)) return writeObject(obj,fw);
+        if (loadLibrary(pluginLibraryName)==LOADED) return writeObject(obj,fw);
 
         // otherwise try simple class name
         if (itr == _classNameWrapperMap.end()) 
@@ -1261,7 +1328,7 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
 
                     // first try the standard nodekit library.
                     std::string nodeKitLibraryName = createLibraryNameForNodeKit(libraryName);
-                    if (loadLibrary(nodeKitLibraryName)) 
+                    if (loadLibrary(nodeKitLibraryName)==LOADED) 
                     {
                         mitr = _objectWrapperMap.find(*aitr);
                     }
@@ -1270,7 +1337,7 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
                     {
                         // otherwise try the osgdb_ plugin library.
                         std::string pluginLibraryName = createLibraryNameForExtension(libraryName);
-                        if (loadLibrary(pluginLibraryName))
+                        if (loadLibrary(pluginLibraryName)==LOADED)
                         {
                             mitr = _objectWrapperMap.find(*aitr);
                         }
@@ -1403,58 +1470,13 @@ ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
             return archive->readObject(fileName,options.get());
         }
     }
-
-    // if filename contains archive
-    // then get archive name
-    // if archive name is not in the cache then do an openArchive on
-    // that archive name
-    // use that archive to read the file.
-
-    if (containsServerAddress(readFunctor._filename))
-    {
-        ReaderWriter* rw = getReaderWriterForExtension("curl");
-        if (rw)
-        {
-            return readFunctor.doRead(*rw);
-        }
-        else
-        {
-            ReaderWriter* rw = getReaderWriterForExtension("net");
-            if (rw)
-            {
-                std::string serverName = getServerAddress(readFunctor._filename);
-                std::string serverFile = getServerFileName(readFunctor._filename);
-                osg::notify(osg::INFO)<<"Contains sever address : "<<serverName<<std::endl;
-                osg::notify(osg::INFO)<<"         file name on server : "<<serverFile<<std::endl;
-
-                if (serverName.empty())
-                {
-                    return ReaderWriter::ReadResult("Warning: Server address invalid.");
-                }
-
-                if (serverFile.empty())
-                {
-                    return ReaderWriter::ReadResult("Warning: Server file name invalid.");
-                }
-
-                std::string& filename = const_cast<std::string&>(readFunctor._filename);
-                filename = serverName+':'+serverFile;
-                return readFunctor.doRead(*rw);
-            }
-            else
-            {
-                return  ReaderWriter::ReadResult("Warning: Could not find the .net plugin to read from server.");
-            }
-            return  ReaderWriter::ReadResult("Warning: Could not find the .curl plugin to read from server.");
-        }
-    }
-    
+ 
     // record the errors reported by readerwriters.
     typedef std::vector<ReaderWriter::ReadResult> Results;
     Results results;
 
     // first attempt to load the file from existing ReaderWriter's
-    AvailableReaderWriterIterator itr(_rwList);
+    AvailableReaderWriterIterator itr(_rwList, _pluginMutex);
     for(;itr.valid();++itr)
     {
         ReaderWriter::ReadResult rr = readFunctor.doRead(*itr);
@@ -1489,12 +1511,16 @@ ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
                 }
             }
 
-            for(ritr=results.begin(); ritr!=results.end(); ++ritr)
+            //If the filename is a URL, don't return FILE_NOT_FOUND until the CURL plugin is given a chance
+            if (!osgDB::containsServerAddress(readFunctor._filename))
             {
-                if (ritr->status()==ReaderWriter::ReadResult::FILE_NOT_FOUND)
+                for(ritr=results.begin(); ritr!=results.end(); ++ritr)
                 {
-                    // osg::notify(osg::NOTICE)<<"Warning: could not find file \""<<readFunctor._filename<<"\""<<std::endl;
-                    return *ritr;
+                    if (ritr->status()==ReaderWriter::ReadResult::FILE_NOT_FOUND)
+                    {
+                        //osg::notify(osg::NOTICE)<<"Warning: could not find file \""<<readFunctor._filename<<"\""<<std::endl;
+                        return *ritr;
+                    }
                 }
             }
         }
@@ -1504,7 +1530,7 @@ ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
 
     // now look for a plug-in to load the file.
     std::string libraryName = createLibraryNameForFile(readFunctor._filename);
-    if (loadLibrary(libraryName))
+    if (loadLibrary(libraryName)!=NOT_LOADED)
     {
         for(;itr.valid();++itr)
         {
@@ -1513,6 +1539,21 @@ ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
             else results.push_back(rr);
         }
     }
+
+    //If the filename contains a server address and wasn't loaded by any of the plugins, try to use the CURL plugin
+    //to download the file and use the stream reading functionality of the plugins to load the file
+    if (containsServerAddress(readFunctor._filename))
+    {
+        ReaderWriter* rw = getReaderWriterForExtension("curl");
+        if (rw)
+        {
+            return readFunctor.doRead(*rw);
+        }
+        else
+        {
+            return  ReaderWriter::ReadResult("Warning: Could not find the .curl plugin to read from server.");
+        }
+    }
     
     if (!results.empty())
     {
@@ -1644,7 +1685,7 @@ ReaderWriter::WriteResult Registry::writeObjectImplementation(const Object& obj,
     Results results;
 
     // first attempt to load the file from existing ReaderWriter's
-    AvailableReaderWriterIterator itr(_rwList);
+    AvailableReaderWriterIterator itr(_rwList, _pluginMutex);
     for(;itr.valid();++itr)
     {
         ReaderWriter::WriteResult rr = itr->writeObject(obj,fileName,options);
@@ -1654,7 +1695,7 @@ ReaderWriter::WriteResult Registry::writeObjectImplementation(const Object& obj,
 
     // now look for a plug-in to save the file.
     std::string libraryName = createLibraryNameForFile(fileName);
-    if (loadLibrary(libraryName))
+    if (loadLibrary(libraryName)==LOADED)
     {
         for(;itr.valid();++itr)
         {
@@ -1697,7 +1738,7 @@ ReaderWriter::WriteResult Registry::writeImageImplementation(const Image& image,
     Results results;
 
     // first attempt to load the file from existing ReaderWriter's
-    AvailableReaderWriterIterator itr(_rwList);
+    AvailableReaderWriterIterator itr(_rwList, _pluginMutex);
     for(;itr.valid();++itr)
     {
         ReaderWriter::WriteResult rr = itr->writeImage(image,fileName,options);
@@ -1709,7 +1750,7 @@ ReaderWriter::WriteResult Registry::writeImageImplementation(const Image& image,
 
     // now look for a plug-in to save the file.
     std::string libraryName = createLibraryNameForFile(fileName);
-    if (loadLibrary(libraryName))
+    if (loadLibrary(libraryName)==LOADED)
     {
         for(;itr.valid();++itr)
         {
@@ -1751,7 +1792,7 @@ ReaderWriter::WriteResult Registry::writeHeightFieldImplementation(const HeightF
     Results results;
 
     // first attempt to load the file from existing ReaderWriter's
-    AvailableReaderWriterIterator itr(_rwList);
+    AvailableReaderWriterIterator itr(_rwList, _pluginMutex);
     for(;itr.valid();++itr)
     {
         ReaderWriter::WriteResult rr = itr->writeHeightField(HeightField,fileName,options);
@@ -1763,7 +1804,7 @@ ReaderWriter::WriteResult Registry::writeHeightFieldImplementation(const HeightF
 
     // now look for a plug-in to save the file.
     std::string libraryName = createLibraryNameForFile(fileName);
-    if (loadLibrary(libraryName))
+    if (loadLibrary(libraryName)==LOADED)
     {
         for(;itr.valid();++itr)
         {
@@ -1794,8 +1835,21 @@ ReaderWriter::WriteResult Registry::writeHeightFieldImplementation(const HeightF
 
 ReaderWriter::ReadResult Registry::readNodeImplementation(const std::string& fileName,const ReaderWriter::Options* options)
 {
+#if 0
+
+    osg::Timer_t startTick = osg::Timer::instance()->tick();
+    ReaderWriter::ReadResult result = readImplementation(ReadNodeFunctor(fileName, options),
+                              options ? (options->getObjectCacheHint()&ReaderWriter::Options::CACHE_NODES)!=0: false);
+    osg::Timer_t endTick = osg::Timer::instance()->tick();
+    osg::notify(osg::NOTICE)<<"time to load "<<fileName<<" "<<osg::Timer::instance()->delta_m(startTick, endTick)<<"ms"<<std::endl;
+    return result;
+
+#else
+
     return readImplementation(ReadNodeFunctor(fileName, options),
                               options ? (options->getObjectCacheHint()&ReaderWriter::Options::CACHE_NODES)!=0: false);
+                              
+#endif
 }
 
 ReaderWriter::WriteResult Registry::writeNodeImplementation(const Node& node,const std::string& fileName,const ReaderWriter::Options* options)
@@ -1805,7 +1859,7 @@ ReaderWriter::WriteResult Registry::writeNodeImplementation(const Node& node,con
     Results results;
 
     // first attempt to write the file from existing ReaderWriter's
-    AvailableReaderWriterIterator itr(_rwList);
+    AvailableReaderWriterIterator itr(_rwList, _pluginMutex);
     for(;itr.valid();++itr)
     {
         ReaderWriter::WriteResult rr = itr->writeNode(node,fileName,options);
@@ -1819,7 +1873,7 @@ ReaderWriter::WriteResult Registry::writeNodeImplementation(const Node& node,con
     std::string libraryName = createLibraryNameForFile(fileName);
 
 
-    if (loadLibrary(libraryName))
+    if (loadLibrary(libraryName)==LOADED)
     {
         for(;itr.valid();++itr)
         {
@@ -1861,7 +1915,7 @@ ReaderWriter::WriteResult Registry::writeShaderImplementation(const Shader& shad
     Results results;
 
     // first attempt to load the file from existing ReaderWriter's
-    AvailableReaderWriterIterator itr(_rwList);
+    AvailableReaderWriterIterator itr(_rwList, _pluginMutex);
     for(;itr.valid();++itr)
     {
         ReaderWriter::WriteResult rr = itr->writeShader(shader,fileName,options);
@@ -1873,7 +1927,7 @@ ReaderWriter::WriteResult Registry::writeShaderImplementation(const Shader& shad
 
     // now look for a plug-in to save the file.
     std::string libraryName = createLibraryNameForFile(fileName);
-    if (loadLibrary(libraryName))
+    if (loadLibrary(libraryName)==LOADED)
     {
         for(;itr.valid();++itr)
         {
@@ -2013,13 +2067,6 @@ void Registry::releaseGLObjects(osg::State* state)
     }
 }
 
-DatabasePager* Registry::getOrCreateDatabasePager()
-{
-    if (!_databasePager) _databasePager = new DatabasePager;
-    
-    return _databasePager.get();
-}
-
 SharedStateManager* Registry::getOrCreateSharedStateManager()
 {
     if (!_sharedStateManager) _sharedStateManager = new SharedStateManager;
