@@ -78,7 +78,9 @@ class PrintVisitor : public NodeVisitor
         virtual void apply(LOD& node)           { apply((Group&)node); }
 
    protected:
-    
+
+        PrintVisitor& operator = (const PrintVisitor&) { return *this; }
+
         std::ostream& _out;
         int _indent;
         int _step;
@@ -91,7 +93,6 @@ class ReaderWriter3DS : public osgDB::ReaderWriter
         ReaderWriter3DS();
 
         virtual const char* className() const { return "3DS Auto Studio Reader"; }
-        virtual bool acceptsExtension(const std::string& extension) const { return osgDB::equalCaseInsensitive(extension,"3ds"); }
 
         virtual ReadResult readNode(const std::string& file, const osgDB::ReaderWriter::Options* options) const;
 
@@ -127,6 +128,8 @@ REGISTER_OSGPLUGIN(3ds, ReaderWriter3DS)
 
 ReaderWriter3DS::ReaderWriter3DS()
 {
+    supportsExtension("3ds","3D Studio model format");
+
     setByteOrder();
 
 #if 0
@@ -714,15 +717,15 @@ osg::Texture2D*  ReaderWriter3DS::ReaderObject::createTexture(Lib3dsTextureMap *
         osg::notify(osg::DEBUG_INFO) << "    LIB3DS_IGNORE_ALPHA "<<((texture->flags)&LIB3DS_IGNORE_ALPHA)<< std::endl;
         osg::notify(osg::DEBUG_INFO) << "    LIB3DS_RGB_TINT     "<<((texture->flags)&LIB3DS_RGB_TINT)<< std::endl;
 
-        osg::Image* osg_image = osgDB::readImageFile(fileName.c_str());
-        if (osg_image==NULL)
+        osg::ref_ptr<osg::Image> osg_image = osgDB::readRefImageFile(fileName.c_str());
+        if (!osg_image)
         {
             osg::notify(osg::NOTICE) << "Warning: Cannot create texture "<<texture->name<< std::endl;
             return NULL;
         }
 
         osg::Texture2D* osg_texture = new osg::Texture2D;
-        osg_texture->setImage(osg_image);
+        osg_texture->setImage(osg_image.get());
 
         // does the texture support transparancy?
         transparancy = ((texture->flags)&LIB3DS_ALPHA_SOURCE)!=0;
@@ -818,7 +821,7 @@ osg::StateSet* ReaderWriter3DS::ReaderObject::createStateSet(Lib3dsMaterial *mat
 //        stateset->setTextureAttribute(0,texenv);
     }
 
-    if (transparency>0.0f || textureTransparancy)
+    if (transparency>0.0f || textureTransparancy || mat->opacity_map.flags!=0)
     {
         stateset->setMode(GL_BLEND,osg::StateAttribute::ON);
         stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
