@@ -86,7 +86,7 @@ public:
 
 /*!
 
-FLTReaderWriter supports importing and exporting OSG scene grqphs
+FLTReaderWriter supports importing and exporting OSG scene graphs
 from/to OpenFlight files.
 
 <table>
@@ -120,7 +120,31 @@ class FLTReaderWriter : public ReaderWriter
     public:
         FLTReaderWriter()
           : _implicitPath( "." )
-        {}
+        {
+            supportsExtension("flt","OpenFlight format");
+            
+            supportsOption("clampToEdge","Import option");
+            supportsOption("keepExternalReferences","Import option");
+            supportsOption("preserveFace","Import option");
+            supportsOption("preserveObject","Import option");
+            supportsOption("dofAnimation","Import option");
+            supportsOption("billboardCenter","Import option");
+            supportsOption("noTextureAlphaForTransparancyBinning","Import option");
+            supportsOption("readObjectRecordData","Import option");
+            supportsOption("noUnitsConversion","Import option");
+            supportsOption("convertToFeet","Import option");
+            supportsOption("convertToInches","Import option");
+            supportsOption("convertToMeters","Import option");
+            supportsOption("convertToKilometers","Import option");
+            supportsOption("convertToNauticalMiles","Import option");
+
+            supportsOption( "version=<ver>", "Export option: Specifies the version of the output OpenFlight file. Supported values include 15.7, 15.8, and 16.1. Default is 16.1. Example: \"version=15.8\"." );
+            supportsOption( "units=<units>", "Export option: Specifies the contents of the Units field of the OpenFliht header record. Valid values include INCHES, FEET, METERS, KILOMETERS, and NATICAL_MILES. Default is METERS. Example: \"units=METERS\"." );
+            supportsOption( "validate", "Export option: If present in the Options string, the plugin does not write an OpenFlight file. Instead, it returns an indication of the scene graph's suitability for OpenFlight export." );
+            supportsOption( "tempDir=<dir>", "Export option: Specifies the directory to use for creation of temporary files. If not specified, the directory is taken from the file name. If the file doesn't contain a path, the current working directory is used. Applications should set this to the name of their app-specific temp directory. If the path contains spaces, use double quotes to ensure correct parsing. Examples: \"tempDir=/tmp\", \"tempDir=\"C:\\My Temp Dir\"." );
+            supportsOption( "lighting=<ON|OFF>", "Export option: Specifies a default enable/disable state for lighting, for Nodes in the exported scene graph that don't set it explicitly. By default, the exporter assumes lighting is enabled (GL_LIGHTING ON). Set this to either ON or OFF. Example: \"lighting=OFF\"." );
+            supportsOption( "stripTextureFilePath", "Export option: If present in the Options string, the exporter strips the path from texture file names, and writes only the texture file name to the FLT Texture Palette. By default, the exporter doesn't strip the path, and the name written to the Texture Palette is taken directly from the osg::Image object referenced by the osg::Texture2D StateAttribute." );
+        }
 
         virtual const char* className() const { return "FLT Reader/Writer"; }
 
@@ -159,7 +183,7 @@ class FLTReaderWriter : public ReaderWriter
 
             // read file
             {
-                std::ifstream istream;
+                osgDB::ifstream istream;
                 istream.imbue(std::locale::classic());
                 istream.open(fileName.c_str(), std::ios::in | std::ios::binary);
 
@@ -284,7 +308,7 @@ class FLTReaderWriter : public ReaderWriter
             }
 
             const int RECORD_HEADER_SIZE = 4;
-            opcode_type continuationOpcode = -1;
+            opcode_type continuationOpcode = INVALID_OP;
             std::string continuationBuffer;
 
             while (fin.good() && !document.done())
@@ -299,6 +323,22 @@ class FLTReaderWriter : public ReaderWriter
                 opcode_type opcode = (opcode_type)dataStream.readUInt16();
                 size_type   size   = (size_type)dataStream.readUInt16();
 
+                // If size == 0, an EOF has probably been reached, i.e. there is nothing 
+                // more to read so we must return.
+                if (size==0)
+                {
+                    // If a header was read, we return it.
+                    // This allows us handle files with empty hierarchies.
+                    if (document.getHeaderNode())
+                    {
+                        return document.getHeaderNode();
+                    }
+                    else // (no valid header)
+                    {
+                        return ReadResult::ERROR_IN_READING_FILE;
+                    }
+                }
+
                 // variable length record complete?
                 if (!continuationBuffer.empty() && opcode!=CONTINUATION_OP)
                 {
@@ -307,7 +347,7 @@ class FLTReaderWriter : public ReaderWriter
                     flt::RecordInputStream recordStream(&sb);
                     recordStream.readRecordBody(continuationOpcode, continuationBuffer.length(), document);
 
-                    continuationOpcode = -1;
+                    continuationOpcode = INVALID_OP;
                     continuationBuffer.clear();
                 }
 
@@ -401,7 +441,7 @@ class FLTReaderWriter : public ReaderWriter
             if (!filePath.empty())
                 _implicitPath = filePath;
 
-            std::ofstream fOut;
+            osgDB::ofstream fOut;
             fOut.open( fileName.c_str(), std::ios::out | std::ios::binary );
             if ( fOut.fail())
             {
