@@ -221,9 +221,10 @@
 #define SUPPORT_CPP 1
 #endif
 
+#include <osgDB/fstream>
+
 #include <string.h>
 #include <stdio.h>
-#include <fstream>
 #include <string>
 
 #include "FlexLexer.h"
@@ -1973,7 +1974,7 @@ yyreduce:
   case 120:
 
     {
-        int n;
+        unsigned int n;
         sscanf( flexer->YYText(), "0x%x", &n );
         yyval.integer = n;
     ;}
@@ -2109,7 +2110,7 @@ yyerrlab:
      token.  */
   goto yyerrlab2;
 
-
+#if 0
 /*----------------------------------------------------.
 | yyerrlab1 -- error raised explicitly by an action.  |
 `----------------------------------------------------*/
@@ -2124,7 +2125,7 @@ yyerrlab1:
 
 
   goto yyerrlab2;
-
+#endif
 
 /*---------------------------------------------------------------.
 | yyerrlab2 -- pop states until the error token can be shifted.  |
@@ -2243,7 +2244,7 @@ bool CameraConfig::parseFile( const std::string &file )
     {
 
         int pd[2];
-        pipe( pd );
+        int result = pipe( pd );
 
         flexer = new yyFlexLexer;
         if( fork() == 0 )
@@ -2251,7 +2252,7 @@ bool CameraConfig::parseFile( const std::string &file )
             // we don't want to read from the pipe in the child, so close it.
             close( pd[0] );
             close( 1 );
-            dup( pd[1] );
+            result = dup( pd[1] );
 
 
             /* This was here to allow reading a config file from stdin.
@@ -2260,7 +2261,7 @@ bool CameraConfig::parseFile( const std::string &file )
                 execlp( cpp_path, "cpp",  "-P", 0L );
             else
             */
-            execlp( cpp_path, "cpp",  "-P", fileName.c_str(), 0L );
+            execlp( cpp_path, "cpp",  "-P", fileName.c_str(), (char *)NULL );
 
             // This should not execute unless an error happens
             perror( "execlp" );
@@ -2269,7 +2270,7 @@ bool CameraConfig::parseFile( const std::string &file )
         {
             close( pd[1]);
             close( 0 );
-            dup( pd[0] );
+            result = dup( pd[0] );
 
             cfg = this;
 
@@ -2283,7 +2284,7 @@ bool CameraConfig::parseFile( const std::string &file )
     else
 #endif
     {
-        std::ifstream ifs(fileName.c_str());
+        osgDB::ifstream ifs(fileName.c_str());
         flexer = new yyFlexLexer(&ifs);
         cfg = this;
         retval = ConfigParser_parse() == 0 ? true : false;
