@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 // reading a design workshop file utility
 // (c) GW Michel, 2001-2003.
@@ -53,11 +54,10 @@ public:
         if (isTextured()) { // shares common textures
             if (!ctx || !tx) { // new texture needed
                 if (fname.length()>0) { 
-                    ctx=osgDB::readImageFile(fname.c_str(),options);
-                    if (ctx) {
+                    ctx=osgDB::readRefImageFile(fname.c_str(),options);
+                    if (ctx.valid()) {
                         ctx->setFileName(fname);
-                        tx=new Texture2D;
-                        tx->setImage(ctx);
+                        tx=new Texture2D(ctx.get());
                         tx->setWrap(Texture2D::WRAP_S, Texture2D::REPEAT);
                         tx->setWrap(Texture2D::WRAP_T, Texture2D::REPEAT);
                     }
@@ -66,8 +66,8 @@ public:
                     dstate->setTextureAttribute(0, texenv );
                 }
             }
-            if (ctx && tx) { // texture exists
-                dstate->setTextureAttributeAndModes(0,tx,osg::StateAttribute::ON);
+            if (ctx.valid() && tx.valid()) { // texture exists
+                dstate->setTextureAttributeAndModes(0,tx.get(),osg::StateAttribute::ON);
             }
         }
     }
@@ -174,8 +174,8 @@ private:
     std::string fname; // picture file
     enum atten {NONE, INVERSE_DIST, INVERSE_SQUARE} atyp;
     float bright,halfIn,halfOut,falloff; // light brightness
-    Image *ctx;
-    Texture2D *tx;
+    osg::ref_ptr<osg::Image> ctx;
+    osg::ref_ptr<osg::Texture2D> tx;
     int _lightnum;
     StateSet *dstate; // used to represent the dw material in OSG
 };
@@ -792,12 +792,13 @@ void _dwobj::buildDrawable(Group *grp, const osgDB::ReaderWriter::Options *optio
 class ReaderWriterDW : public osgDB::ReaderWriter
 {
     public:
-        virtual const char* className() const { return "Design Workshop Database Reader"; }
-
-        virtual bool acceptsExtension(const std::string& extension) const
+    
+        ReaderWriterDW()
         {
-            return osgDB::equalCaseInsensitive(extension,"dw");
+            supportsExtension("dw","Designer Workbench model format");
         }
+    
+        virtual const char* className() const { return "Design Workshop Database Reader"; }
 
         virtual ReadResult readNode(const std::string& file,const osgDB::ReaderWriter::Options* options) const
         {
@@ -827,7 +828,7 @@ class ReaderWriterDW : public osgDB::ReaderWriter
 
             FILE *fp;
 
-            if( (fp = fopen( fileName.c_str(), "r" )) == (FILE *)0L )
+            if( (fp = osgDB::fopen( fileName.c_str(), "r" )) == (FILE *)0L )
             {
                 return std::string("Unable to open file \""+fileName+"\"");
             }
