@@ -22,6 +22,7 @@
 #include "BlendColor.h"
 #include "Stencil.h"
 #include "BlendFunc.h"
+#include "BlendEquation.h"
 #include "Material.h"
 #include "CullFace.h"
 #include "ColorMask.h"
@@ -32,6 +33,7 @@
 #include "ShadeModel.h"
 #include "Point.h"
 #include "LineWidth.h"
+#include "LineStipple.h"
 #include "Texture1D.h"
 #include "Texture2D.h"
 #include "Texture3D.h"
@@ -52,8 +54,11 @@
 #include "Viewport.h"
 #include "Scissor.h"
 #include "Image.h"
+#include "ImageSequence.h"
 #include "PointSprite.h"
 #include "Multisample.h"
+#include "Fog.h"
+#include "Light.h"
 
 #include "Group.h"
 #include "MatrixTransform.h"
@@ -82,6 +87,13 @@
 #include "VisibilityGroup.h"
 
 #include "MultiTextureControl.h"
+#include "ShapeAttributeList.h"
+#include "Effect.h"
+#include "AnisotropicLighting.h"
+#include "BumpMapping.h"
+#include "Cartoon.h"
+#include "Scribe.h"
+#include "SpecularHighlights.h"
 
 #include "Geometry.h"
 #include "ShapeDrawable.h"
@@ -95,44 +107,20 @@
 #include "ImageLayer.h"
 #include "HeightFieldLayer.h"
 #include "CompositeLayer.h"
+#include "SwitchLayer.h"
 
 #include <osg/Notify>
 #include <osg/io_utils>
 #include <osgDB/FileUtils>
+#include <osgDB/fstream>
 
-#include <fstream>
+#include <stdlib.h>
 #include <sstream>
 
 using namespace ive;
 
 
-void DataOutputStream::setOptions(const osgDB::ReaderWriter::Options* options) 
-{ 
-    _options = options; 
-
-    if (_options.get())
-    {
-        if(_options->getOptionString().find("noTexturesInIVEFile")!=std::string::npos) {
-            setIncludeImageMode(IMAGE_REFERENCE_FILE);
-        } else if(_options->getOptionString().find("includeImageFileInIVEFile")!=std::string::npos) {
-            setIncludeImageMode(IMAGE_INCLUDE_FILE);
-        } else if(_options->getOptionString().find("compressImageData")!=std::string::npos) {
-            setIncludeImageMode(IMAGE_COMPRESS_DATA);
-        }
-        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeImageMode()=" << getIncludeImageMode() << std::endl;
-
-        setIncludeExternalReferences(_options->getOptionString().find("inlineExternalReferencesInIVEFile")!=std::string::npos);
-        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeExternalReferences()=" << getIncludeExternalReferences() << std::endl;
-
-        setWriteExternalReferenceFiles(_options->getOptionString().find("noWriteExternalReferenceFiles")==std::string::npos);
-        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setWriteExternalReferenceFiles()=" << getWriteExternalReferenceFiles() << std::endl;
-
-        setUseOriginalExternalReferences(_options->getOptionString().find("useOriginalExternalReferences")!=std::string::npos);
-        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setUseOriginalExternalReferences()=" << getUseOriginalExternalReferences() << std::endl;
-    }
-}
-
-DataOutputStream::DataOutputStream(std::ostream * ostream)
+DataOutputStream::DataOutputStream(std::ostream * ostream, const osgDB::ReaderWriter::Options* options)
 {
     _verboseOutput = false;
 
@@ -141,40 +129,190 @@ DataOutputStream::DataOutputStream(std::ostream * ostream)
     _includeExternalReferences     = false;
     _writeExternalReferenceFiles   = false;
     _useOriginalExternalReferences = true;
+    _maximumErrorToSizeRatio       = 0.001;
+
+    _options = options;
+
+    _compressionLevel = 0;
+
+    if (_options.get())
+    {
+        std::string optionsString = _options->getOptionString();
     
-    
-    _ostream = ostream;
+        if(optionsString.find("noTexturesInIVEFile")!=std::string::npos) {
+            setIncludeImageMode(IMAGE_REFERENCE_FILE);
+        } else if(optionsString.find("includeImageFileInIVEFile")!=std::string::npos) {
+            setIncludeImageMode(IMAGE_INCLUDE_FILE);
+        } else if(optionsString.find("compressImageData")!=std::string::npos) {
+            setIncludeImageMode(IMAGE_COMPRESS_DATA);
+        }
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeImageMode()=" << getIncludeImageMode() << std::endl;
+
+        setIncludeExternalReferences(optionsString.find("inlineExternalReferencesInIVEFile")!=std::string::npos);
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeExternalReferences()=" << getIncludeExternalReferences() << std::endl;
+
+        setWriteExternalReferenceFiles(optionsString.find("noWriteExternalReferenceFiles")==std::string::npos);
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setWriteExternalReferenceFiles()=" << getWriteExternalReferenceFiles() << std::endl;
+
+        setUseOriginalExternalReferences(optionsString.find("useOriginalExternalReferences")!=std::string::npos);
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setUseOriginalExternalReferences()=" << getUseOriginalExternalReferences() << std::endl;
+
+        _compressionLevel =  (optionsString.find("compressed")!=std::string::npos) ? 1 : 0;
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream._compressionLevel=" << _compressionLevel << std::endl;
+
+        std::string::size_type terrainErrorPos = optionsString.find("TerrainMaximumErrorToSizeRatio=");
+        if (terrainErrorPos!=std::string::npos)
+        {
+            std::string::size_type endOfToken = optionsString.find_first_of('=', terrainErrorPos);
+            std::string::size_type endOfNumber = optionsString.find_first_of(' ', endOfToken);
+            std::string::size_type numOfCharInNumber = (endOfNumber != std::string::npos) ? 
+                    endOfNumber-endOfToken-1 :
+                    optionsString.size()-endOfToken-1;
+
+            if (numOfCharInNumber>0)
+            {
+                std::string numberString = optionsString.substr(endOfToken+1, numOfCharInNumber);
+                _maximumErrorToSizeRatio = atof(numberString.c_str());
+                
+                osg::notify(osg::DEBUG_INFO)<<"TerrainMaximumErrorToSizeRatio = "<<_maximumErrorToSizeRatio<<std::endl;
+            }
+            else
+            {
+                osg::notify(osg::DEBUG_INFO)<<"Error no value to TerrainMaximumErrorToSizeRatio assigned"<<std::endl;
+            }
+        }
+    }
+
+    #ifndef USE_ZLIB
+    if (_compressionLevel>0)
+    {
+        osg::notify(osg::NOTICE) << "Compression not supported in this .ive version." << std::endl;
+        _compressionLevel = 0;
+    }
+    #endif
+
+    _output_ostream = _ostream = ostream;
+
     if(!_ostream)
         throw Exception("DataOutputStream::DataOutputStream(): null pointer exception in argument.");
+
     writeUInt(ENDIAN_TYPE) ;
     writeUInt(getVersion());
+    
+    writeInt(_compressionLevel);
+
+    if (_compressionLevel>0)
+    {
+    
+        _ostream = &_compressionStream;        
+    }
 }
 
-DataOutputStream::~DataOutputStream(){}
+DataOutputStream::~DataOutputStream()
+{
+    if (_compressionLevel>0)
+    { 
+        _ostream = _output_ostream;
+
+        std::string compressionString(_compressionStream.str());
+        writeUInt(compressionString.size());
+        
+        compress(*_output_ostream, compressionString);
+    }
+}
+
+#ifdef USE_ZLIB
+
+#include <zlib.h>
+
+#define CHUNK 16384
+bool DataOutputStream::compress(std::ostream& fout, const std::string& source) const
+{
+    int ret, flush = Z_FINISH;
+    unsigned have;
+    z_stream strm;
+    unsigned char out[CHUNK];
+    
+    int level = 6;
+    int stategy = Z_DEFAULT_STRATEGY; // looks to be the best for .osg/.ive files
+    //int stategy = Z_FILTERED;
+    //int stategy = Z_HUFFMAN_ONLY;
+    //int stategy = Z_RLE;
+
+    /* allocate deflate state */
+    strm.zalloc = Z_NULL;
+    strm.zfree = Z_NULL;
+    strm.opaque = Z_NULL;
+    ret = deflateInit2(&strm, 
+                       level,
+                       Z_DEFLATED,
+                       15+16, // +16 to use gzip encoding
+                       8, // default
+                       stategy);
+    if (ret != Z_OK)
+    return false;
+
+    strm.avail_in = source.size();
+    strm.next_in = (Bytef*)(&(*source.begin()));
+
+    /* run deflate() on input until output buffer not full, finish
+       compression if all of source has been read in */
+    do {
+        strm.avail_out = CHUNK;
+        strm.next_out = out;
+        ret = deflate(&strm, flush);    /* no bad return value */
+
+        if (ret == Z_STREAM_ERROR)
+        {
+            osg::notify(osg::NOTICE)<<"Z_STREAM_ERROR"<<std::endl;
+            return false;
+        }
+
+        have = CHUNK - strm.avail_out;
+
+        if (have>0) fout.write((const char*)out, have);
+        
+        if (fout.fail())
+        {
+            (void)deflateEnd(&strm);
+            return false;
+        }
+    } while (strm.avail_out == 0);
+
+    /* clean up and return */
+    (void)deflateEnd(&strm);
+    return true;
+}
+#else
+bool DataOutputStream::compress(std::ostream& fout, const std::string& source) const
+{
+    return false;
+}
+#endif
 
 void DataOutputStream::writeBool(bool b)
 {
     char c = b?1:0;
     _ostream->write(&c, CHARSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeBool() ["<<(int)c<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeChar(char c){
     _ostream->write(&c, CHARSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeChar() ["<<(int)c<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeUChar(unsigned char c){
     _ostream->write((char*)&c, CHARSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeUChar() ["<<(int)c<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeUShort(unsigned short s){
     _ostream->write((char*)&s, SHORTSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeUShort() ["<<s<<"]"<<std::endl;
 }
 
@@ -186,57 +324,57 @@ void DataOutputStream::writeShort(short s){
 
 void DataOutputStream::writeUInt(unsigned int s){
     _ostream->write((char*)&s, INTSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeUInt() ["<<s<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeInt(int i){
     _ostream->write((char*)&i, INTSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeInt() ["<<i<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeFloat(float f){
     _ostream->write((char*)&f, FLOATSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeFloat() ["<<f<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeLong(long l){
     _ostream->write((char*)&l, LONGSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeLong() ["<<l<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeULong(unsigned long l){
     _ostream->write((char*)&l, LONGSIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeULong() ["<<l<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeDouble(double d){
     _ostream->write((char*)&d, DOUBLESIZE);
-    
+
     if (_verboseOutput) std::cout<<"read/writeDouble() ["<<d<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeString(const std::string& s){
     writeInt(s.size());
     _ostream->write(s.c_str(), s.size());
-    
+
     if (_verboseOutput) std::cout<<"read/writeString() ["<<s<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeCharArray(const char* data, int size){
     _ostream->write(data, size);
-    
+
     if (_verboseOutput) std::cout<<"read/writeCharArray() ["<<data<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeVec2(const osg::Vec2& v){
     writeFloat(v.x());
     writeFloat(v.y());
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec2() ["<<v<<"]"<<std::endl;
 }
 
@@ -244,7 +382,7 @@ void DataOutputStream::writeVec3(const osg::Vec3& v){
     writeFloat(v.x());
     writeFloat(v.y());
     writeFloat(v.z());
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec3() ["<<v<<"]"<<std::endl;
 }
 
@@ -253,14 +391,14 @@ void DataOutputStream::writeVec4(const osg::Vec4& v){
     writeFloat(v.y());
     writeFloat(v.z());
     writeFloat(v.w());
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec4() ["<<v<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeVec2d(const osg::Vec2d& v){
     writeDouble(v.x());
     writeDouble(v.y());
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec2() ["<<v<<"]"<<std::endl;
 }
 
@@ -268,7 +406,7 @@ void DataOutputStream::writeVec3d(const osg::Vec3d& v){
     writeDouble(v.x());
     writeDouble(v.y());
     writeDouble(v.z());
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec3d() ["<<v<<"]"<<std::endl;
 }
 
@@ -277,7 +415,7 @@ void DataOutputStream::writeVec4d(const osg::Vec4d& v){
     writeDouble(v.y());
     writeDouble(v.z());
     writeDouble(v.w());
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec4d() ["<<v<<"]"<<std::endl;
 }
 
@@ -287,7 +425,7 @@ void DataOutputStream::writePlane(const osg::Plane& v)
     writeDouble(v[1]);
     writeDouble(v[2]);
     writeDouble(v[3]);
-    
+
     if (_verboseOutput) std::cout<<"read/writePlane() ["<<v<<"]"<<std::endl;
 }
 
@@ -296,13 +434,13 @@ void DataOutputStream::writeVec4ub(const osg::Vec4ub& v){
     writeChar(v.g());
     writeChar(v.b());
     writeChar(v.a());
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec4ub() ["<<v<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeVec2b(const osg::Vec2b& v){
     writeChar(v.r());
-    writeChar(v.g());    
+    writeChar(v.g());
 
     if (_verboseOutput) std::cout<<"read/writeVec2b() ["<<v<<"]"<<std::endl;
 }
@@ -329,7 +467,7 @@ void DataOutputStream::writeQuat(const osg::Quat& q){
     writeFloat(q.y());
     writeFloat(q.z());
     writeFloat(q.w());
-    
+
     if (_verboseOutput) std::cout<<"read/writeQuat() ["<<q<<"]"<<std::endl;
 }
 
@@ -342,13 +480,13 @@ void DataOutputStream::writeBinding(osg::Geometry::AttributeBinding b){
         case osg::Geometry::BIND_PER_VERTEX:        writeChar((char) 4); break;
         default: throw Exception("Unknown binding in DataOutputStream::writeBinding()");
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeBinding() ["<<b<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeArray(const osg::Array* a){
     switch(a->getType()){
-        case osg::Array::IntArrayType: 
+        case osg::Array::IntArrayType:
             writeChar((char)0);
             writeIntArray(static_cast<const osg::IntArray*>(a));
             break;
@@ -372,15 +510,15 @@ void DataOutputStream::writeArray(const osg::Array* a){
             writeChar((char)5);
             writeFloatArray(static_cast<const osg::FloatArray*>(a));
             break;
-        case osg::Array::Vec2ArrayType: 
+        case osg::Array::Vec2ArrayType:
             writeChar((char)6);
             writeVec2Array(static_cast<const osg::Vec2Array*>(a));
             break;
-        case osg::Array::Vec3ArrayType: 
+        case osg::Array::Vec3ArrayType:
             writeChar((char)7);
             writeVec3Array(static_cast<const osg::Vec3Array*>(a));
             break;
-         case osg::Array::Vec4ArrayType: 
+         case osg::Array::Vec4ArrayType:
             writeChar((char)8);
             writeVec4Array(static_cast<const osg::Vec4Array*>(a));
             break;
@@ -408,15 +546,15 @@ void DataOutputStream::writeArray(const osg::Array* a){
              writeChar((char)14);
              writeVec4bArray(static_cast<const osg::Vec4bArray*>(a));
              break;
-         case osg::Array::Vec2dArrayType: 
+         case osg::Array::Vec2dArrayType:
              writeChar((char)15);
              writeVec2dArray(static_cast<const osg::Vec2dArray*>(a));
              break;
-         case osg::Array::Vec3dArrayType: 
+         case osg::Array::Vec3dArrayType:
              writeChar((char)16);
              writeVec3dArray(static_cast<const osg::Vec3dArray*>(a));
              break;
-          case osg::Array::Vec4dArrayType: 
+          case osg::Array::Vec4dArrayType:
              writeChar((char)17);
              writeVec4dArray(static_cast<const osg::Vec4dArray*>(a));
              break;
@@ -432,62 +570,173 @@ void DataOutputStream::writeIntArray(const osg::IntArray* a)
     for(int i =0; i<size ;i++){
         writeInt(a->index(i));
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeIntArray() ["<<size<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeUByteArray(const osg::UByteArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeChar((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeUByteArray() ["<<size<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeUShortArray(const osg::UShortArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeUShort((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeUShortArray() ["<<size<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeUIntArray(const osg::UIntArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeInt((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeUIntArray() ["<<size<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeVec4ubArray(const osg::Vec4ubArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeVec4ub((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec4ubArray() ["<<size<<"]"<<std::endl;
 }
 
+void DataOutputStream::writePackedFloatArray(const osg::FloatArray* a, float maxError)
+{
+    int size = a->getNumElements();
+    writeInt(size);
+    if (size==0) return;
+    
+    float minValue = (*a)[0];
+    float maxValue = minValue;    
+    for(int i=1; i<size; ++i)
+    {
+        if ((*a)[i]<minValue) minValue = (*a)[i];
+        if ((*a)[i]>maxValue) maxValue = (*a)[i];
+    }
+    
+    if (minValue==maxValue)
+    {
+        osg::notify(osg::DEBUG_INFO)<<"Writing out "<<size<<" same values "<<minValue<<std::endl;
+
+        writeBool(true);
+        writeFloat(minValue);
+        return;
+    }
+
+    writeBool(false);
+    
+    int packingSize = 4;
+    if (maxError>0.0f)
+    {
+    
+        //float byteError = 0.0f;
+        float byteMultiplier = 255.0f/(maxValue-minValue);
+        float byteInvMultiplier = 1.0f/byteMultiplier;
+
+        //float shortError = 0.0f;
+        float shortMultiplier = 65535.0f/(maxValue-minValue);
+        float shortInvMultiplier = 1.0f/shortMultiplier;
+
+        float max_error_byte = 0.0f;
+        float max_error_short = 0.0f;
+
+        for(int i=0; i<size; ++i)
+        {
+            float value = (*a)[i];
+            unsigned char byteValue = (unsigned char)((value-minValue)*byteMultiplier);
+            unsigned short shortValue = (unsigned short)((value-minValue)*shortMultiplier);
+            float value_byte = minValue + float(byteValue)*byteInvMultiplier;
+            float value_short = minValue + float(shortValue)*shortInvMultiplier;
+
+            float error_byte = fabsf(value_byte - value);
+            float error_short = fabsf(value_short - value);
+
+            if (error_byte>max_error_byte) max_error_byte = error_byte;
+            if (error_short>max_error_short) max_error_short = error_short;
+        }
+
+        osg::notify(osg::DEBUG_INFO)<<"maxError "<<maxError<<std::endl;
+        osg::notify(osg::DEBUG_INFO)<<"Values to write "<<size<<" max_error_byte = "<<max_error_byte<<" max_error_short="<<max_error_short<<std::endl;
+
+
+        if (max_error_byte < maxError) packingSize = 1;
+        else if (max_error_short < maxError) packingSize = 2;
+
+        osg::notify(osg::DEBUG_INFO)<<"packingSize "<<packingSize<<std::endl;
+
+    }
+
+    if (packingSize==1)
+    {
+        writeInt(1);
+        
+        writeFloat(minValue);
+        writeFloat(maxValue);
+        
+        float byteMultiplier = 255.0f/(maxValue-minValue);
+
+        for(int i=0; i<size; ++i)
+        {
+            unsigned char currentValue = (unsigned char)(((*a)[i]-minValue)*byteMultiplier);
+            writeUChar(currentValue);
+        }
+    }
+    else if (packingSize==2)
+    {
+        writeInt(2);
+
+        writeFloat(minValue);
+        writeFloat(maxValue);
+        
+        float shortMultiplier = 65535.0f/(maxValue-minValue);
+
+        for(int i=0; i<size; ++i)
+        {
+            unsigned short currentValue = (unsigned short)(((*a)[i]-minValue)*shortMultiplier);
+            writeUShort(currentValue);
+        }
+    }
+    else
+    {            
+        writeInt(4);
+
+        for(int i=0; i<size; ++i)
+        {
+            writeFloat((*a)[i]);
+        }
+        
+    }
+
+    if (_verboseOutput) std::cout<<"read/writePackedFloatArray() ["<<size<<"]"<<std::endl;
+}
+
+
 void DataOutputStream::writeFloatArray(const osg::FloatArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeFloat((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeFloatArray() ["<<size<<"]"<<std::endl;
 }
 
@@ -499,7 +748,7 @@ void DataOutputStream::writeVec2Array(const osg::Vec2Array* a)
     for(int i=0;i<size;i++){
         writeVec2((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec2Array() ["<<size<<"]"<<std::endl;
 }
 
@@ -510,7 +759,7 @@ void DataOutputStream::writeVec3Array(const osg::Vec3Array* a)
     for(int i = 0; i < size; i++){
         writeVec3((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec3Array() ["<<size<<"]"<<std::endl;
 }
 
@@ -521,13 +770,13 @@ void DataOutputStream::writeVec4Array(const osg::Vec4Array* a)
     for(int i=0;i<size;i++){
         writeVec4((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec4Array() ["<<size<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeVec2sArray(const osg::Vec2sArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeShort((*a)[i].x());
@@ -539,7 +788,7 @@ void DataOutputStream::writeVec2sArray(const osg::Vec2sArray* a)
 
 void DataOutputStream::writeVec3sArray(const osg::Vec3sArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeShort((*a)[i].x());
@@ -552,7 +801,7 @@ void DataOutputStream::writeVec3sArray(const osg::Vec3sArray* a)
 
 void DataOutputStream::writeVec4sArray(const osg::Vec4sArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeShort((*a)[i].x());
@@ -566,7 +815,7 @@ void DataOutputStream::writeVec4sArray(const osg::Vec4sArray* a)
 
 void DataOutputStream::writeVec2bArray(const osg::Vec2bArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeVec2b((*a)[i]);
@@ -577,7 +826,7 @@ void DataOutputStream::writeVec2bArray(const osg::Vec2bArray* a)
 
 void DataOutputStream::writeVec3bArray(const osg::Vec3bArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeVec3b((*a)[i]);
@@ -588,7 +837,7 @@ void DataOutputStream::writeVec3bArray(const osg::Vec3bArray* a)
 
 void DataOutputStream::writeVec4bArray(const osg::Vec4bArray* a)
 {
-    int size = a->getNumElements(); 
+    int size = a->getNumElements();
     writeInt(size);
     for(int i =0; i<size ;i++){
         writeVec4b((*a)[i]);
@@ -604,7 +853,7 @@ void DataOutputStream::writeVec2dArray(const osg::Vec2dArray* a)
     for(int i=0;i<size;i++){
         writeVec2d((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec2dArray() ["<<size<<"]"<<std::endl;
 }
 
@@ -615,7 +864,7 @@ void DataOutputStream::writeVec3dArray(const osg::Vec3dArray* a)
     for(int i = 0; i < size; i++){
         writeVec3d((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec3dArray() ["<<size<<"]"<<std::endl;
 }
 
@@ -626,7 +875,7 @@ void DataOutputStream::writeVec4dArray(const osg::Vec4dArray* a)
     for(int i=0;i<size;i++){
         writeVec4d((*a)[i]);
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeVec4dArray() ["<<size<<"]"<<std::endl;
 }
 
@@ -639,7 +888,7 @@ void DataOutputStream::writeMatrixf(const osg::Matrixf& mat)
             writeFloat(mat(r,c));
         }
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeMatrix() ["<<mat<<"]"<<std::endl;
 }
 
@@ -652,7 +901,7 @@ void DataOutputStream::writeMatrixd(const osg::Matrixd& mat)
             writeDouble(mat(r,c));
         }
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writeMatrix() ["<<mat<<"]"<<std::endl;
 }
 
@@ -719,6 +968,9 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::BlendFunc*>(attribute)){
             ((ive::BlendFunc*)(attribute))->write(this);
         }
+        else if(dynamic_cast<const osg::BlendEquation*>(attribute)){
+            ((ive::BlendEquation*)(attribute))->write(this);
+        }
         else if(dynamic_cast<const osg::Depth*>(attribute)){
             ((ive::Depth*)(attribute))->write(this);
         }
@@ -761,8 +1013,9 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::LineWidth*>(attribute)){
             ((ive::LineWidth*)(attribute))->write(this);
         }
-        else if(dynamic_cast<const osg::LineWidth*>(attribute)){
-            ((ive::LineWidth*)(attribute))->write(this);
+        // This is a LineStipple
+        else if(dynamic_cast<const osg::LineStipple*>(attribute)){
+            ((ive::LineStipple*)(attribute))->write(this);
         }
         // This is a Texture1D
         else if(dynamic_cast<const osg::Texture1D*>(attribute)){
@@ -828,6 +1081,14 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::Multisample*>(attribute)){
             ((ive::Multisample*)(attribute))->write(this);
         }
+        // This is a Fog
+        else if(dynamic_cast<const osg::Fog*>(attribute)){
+            ((ive::Fog*)(attribute))->write(this);
+        }
+        // This is a Light
+        else if(dynamic_cast<const osg::Light*>(attribute)){
+            ((ive::Light*)(attribute))->write(this);
+        }
 
         else{
             std::string className = attribute->className();
@@ -915,7 +1176,7 @@ void DataOutputStream::writeDrawable(const osg::Drawable* drawable)
 
         // write the id.
         writeInt(id);
-        
+
         if(dynamic_cast<const osg::Geometry*>(drawable))
             ((ive::Geometry*)(drawable))->write(this);
         else if(dynamic_cast<const osg::ShapeDrawable*>(drawable))
@@ -950,7 +1211,7 @@ void DataOutputStream::writeShape(const osg::Shape* shape)
 
         // write the id.
         writeInt(id);
-        
+
         if(dynamic_cast<const osg::Sphere*>(shape))
             ((ive::Sphere*)(shape))->write(this);
         else if(dynamic_cast<const osg::Box*>(shape))
@@ -1060,6 +1321,24 @@ void DataOutputStream::writeNode(const osg::Node* node)
         else if(dynamic_cast<const osgFX::MultiTextureControl*>(node)){
             ((ive::MultiTextureControl*)(node))->write(this);
         }
+
+
+        else if(dynamic_cast<const osgFX::AnisotropicLighting*>(node)){
+            ((ive::AnisotropicLighting*)(node))->write(this);
+        }
+        else if(dynamic_cast<const osgFX::BumpMapping*>(node)){
+            ((ive::BumpMapping*)(node))->write(this);
+        }
+        else if(dynamic_cast<const osgFX::Cartoon*>(node)){
+            ((ive::Cartoon*)(node))->write(this);
+        }
+        else if(dynamic_cast<const osgFX::Scribe*>(node)){
+            ((ive::Scribe*)(node))->write(this);
+        }
+        else if(dynamic_cast<const osgFX::SpecularHighlights*>(node)){
+            ((ive::SpecularHighlights*)(node))->write(this);
+        }
+
         else if(dynamic_cast<const osgTerrain::TerrainTile*>(node)){
             ((ive::TerrainTile*)(node))->write(this);
         }
@@ -1082,6 +1361,48 @@ void DataOutputStream::writeNode(const osg::Node* node)
     }
 }
 
+IncludeImageMode DataOutputStream::getIncludeImageMode(const osg::Image* image) const
+{
+    if (image)
+    {
+        if (image->getWriteHint()==osg::Image::STORE_INLINE) 
+        {
+            return IMAGE_INCLUDE_DATA;
+        }
+        else if (image->getWriteHint()==osg::Image::EXTERNAL_FILE) 
+        {
+            return IMAGE_REFERENCE_FILE;
+        }
+    }
+    return getIncludeImageMode();
+}
+
+
+void DataOutputStream::writeImage(osg::Image *image)
+{
+    IncludeImageMode mode = getIncludeImageMode(image);
+    
+    if ( getVersion() >= VERSION_0029)
+    {
+        osg::ImageSequence* is = dynamic_cast<osg::ImageSequence*>(image);
+        if (is)
+        {
+            ((ive::ImageSequence*)(is))->write(this);
+        }
+        else
+        {
+            writeInt(IVEIMAGE);
+            writeChar(mode);
+            writeImage(mode,image);
+        }
+    }
+    else
+    {
+        writeChar(mode);
+        writeImage(mode,image);
+    }
+}
+
 void DataOutputStream::writeImage(IncludeImageMode mode, osg::Image *image)
 {
     switch(mode) {
@@ -1096,15 +1417,15 @@ void DataOutputStream::writeImage(IncludeImageMode mode, osg::Image *image)
             if (image && !(image->getFileName().empty())){
                 writeString(image->getFileName());
             }
-            else{ 
+            else{
                 writeString("");
-            }    
+            }
             break;
         case IMAGE_INCLUDE_FILE:
             // Include image file in stream
             if(image && !(image->getFileName().empty())) {
                 std::string fullPath = osgDB::findDataFile(image->getFileName(),_options.get());
-                std::ifstream infile(fullPath.c_str(), std::ios::in | std::ios::binary);
+                osgDB::ifstream infile(fullPath.c_str(), std::ios::in | std::ios::binary);
                 if(infile) {
 
                     //Write filename
@@ -1130,25 +1451,25 @@ void DataOutputStream::writeImage(IncludeImageMode mode, osg::Image *image)
 
                     //Close file
                     infile.close();
-                   
+
                 } else {
                     writeString("");
                     writeInt(0);
                 }
             }
-            else{ 
+            else{
                 writeString("");
                 writeInt(0);
-            }    
+            }
             break;
         case IMAGE_COMPRESS_DATA:
             if(image)
             {
                 //Get ReaderWriter for jpeg images
-                
+
                 std::string extension = "png";
                 if (image->getPixelFormat()==GL_RGB) extension = "jpg";
-                
+
                 osgDB::ReaderWriter* writer = osgDB::Registry::instance()->getReaderWriterForExtension(extension);
 
                 if(writer)
@@ -1167,11 +1488,11 @@ void DataOutputStream::writeImage(IncludeImageMode mode, osg::Image *image)
                         // 1 - Same code can be used to read in as with IMAGE_INCLUDE_FILE mode
                         // 2 - Maybe in future version user can specify which format to use
                         writeString(std::string(".")+extension); //Need to add dot so osgDB::getFileExtension will work
-                        
+
                         //Write size of stream
                         int size = outputStream.tellp();
                         writeInt(size);
-                       
+
                         //Write stream
                         writeCharArray(outputStream.str().c_str(),size);
 
@@ -1216,7 +1537,7 @@ void DataOutputStream::writeLayer(const osgTerrain::Layer* layer)
 
         // write the id.
         writeInt(id);
-        
+
         if (dynamic_cast<const osgTerrain::HeightFieldLayer*>(layer))
         {
             ((ive::HeightFieldLayer*)(layer))->write(this);
@@ -1225,6 +1546,10 @@ void DataOutputStream::writeLayer(const osgTerrain::Layer* layer)
         {
             ((ive::ImageLayer*)(layer))->write(this);
         }
+        else if (dynamic_cast<const osgTerrain::SwitchLayer*>(layer))
+        {
+            ((ive::SwitchLayer*)(layer))->write(this);
+        }
         else if (dynamic_cast<const osgTerrain::CompositeLayer*>(layer))
         {
             ((ive::CompositeLayer*)(layer))->write(this);
@@ -1284,3 +1609,49 @@ void DataOutputStream::writeLocator(const osgTerrain::Locator* locator)
 
     }
 }
+
+void DataOutputStream::writeObject(const osg::Object* object)
+{
+    const osg::Node* node = dynamic_cast<const osg::Node*>(object);
+    if (node) 
+    {
+        writeInt(IVENODE);
+        writeNode(node);
+        return;
+    }
+
+    const osg::StateSet* stateset = dynamic_cast<const osg::StateSet*>(object);
+    if (stateset) 
+    {
+        writeInt(IVESTATESET);
+        writeStateSet(stateset);
+        return;
+    }
+
+    const osg::StateAttribute* sa = dynamic_cast<const osg::StateAttribute*>(object);
+    if (sa) 
+    {
+        writeInt(IVESTATEATTRIBUTE);
+        writeStateAttribute(sa);
+        return;
+    }
+
+    const osg::Drawable* drawable = dynamic_cast<const osg::Drawable*>(object);
+    if (drawable) 
+    {
+        writeInt(IVEDRAWABLE);
+        writeDrawable(drawable);
+        return;
+    }
+
+    const osgSim::ShapeAttributeList* sal = dynamic_cast<const osgSim::ShapeAttributeList*>(object);
+    if (sal) 
+    {
+        writeInt(IVESHAPEATTRIBUTELIST);
+        ((ive::ShapeAttributeList*)sal)->write(this);
+        return;
+    }
+
+    // fallback, osg::Object type not supported, so can't write out
+    writeInt(-1);
+}
