@@ -228,7 +228,7 @@ void SceneView::setDefaults(unsigned int options)
  
     _renderInfo.setState(new State);
     
-    _rendergraph = new StateGraph;
+    _stateGraph = new StateGraph;
     _renderStage = new RenderStage;
 
 
@@ -256,7 +256,7 @@ void SceneView::setDefaults(unsigned int options)
 
     _cullVisitor = CullVisitor::create();
 
-    _cullVisitor->setStateGraph(_rendergraph.get());
+    _cullVisitor->setStateGraph(_stateGraph.get());
     _cullVisitor->setRenderStage(_renderStage.get());
 
     _globalStateSet->setGlobalDefaults();
@@ -542,17 +542,154 @@ osg::Matrixd SceneView::computeRightEyeViewImplementation(const osg::Matrixd& vi
                        -es,0.0,0.0,1.0);
 }
 
+void SceneView::computeLeftEyeViewport(const osg::Viewport *viewport)
+{
+    if(!viewport) return;
+
+    if(!_viewportLeft.valid()) _viewportLeft = new osg::Viewport;
+
+    switch(_displaySettings->getStereoMode())
+    {
+
+        case(osg::DisplaySettings::HORIZONTAL_SPLIT):
+        {
+            Viewport::value_type separation = _displaySettings->getSplitStereoHorizontalSeparation();
+
+            if (_displaySettings->getSplitStereoHorizontalEyeMapping()==osg::DisplaySettings::LEFT_EYE_LEFT_VIEWPORT)
+            {
+                Viewport::value_type left_half_width = (viewport->width()-separation)/2;
+                _viewportLeft->setViewport(viewport->x(),viewport->y(),left_half_width,viewport->height());
+            }
+            else
+            {
+                Viewport::value_type right_half_begin = (viewport->width()+separation)/2;
+                Viewport::value_type right_half_width = viewport->width()-right_half_begin;
+                _viewportLeft->setViewport(viewport->x()+right_half_begin,viewport->y(),right_half_width,viewport->height());
+            }
+        }
+        break;
+
+        case(osg::DisplaySettings::VERTICAL_SPLIT):
+        {
+            Viewport::value_type separation = _displaySettings->getSplitStereoVerticalSeparation();
+
+            if (_displaySettings->getSplitStereoVerticalEyeMapping()==osg::DisplaySettings::LEFT_EYE_TOP_VIEWPORT)
+            {
+                Viewport::value_type top_half_begin = (viewport->height()+separation)/2;
+                Viewport::value_type top_half_height = viewport->height()-top_half_begin;
+                _viewportLeft->setViewport(viewport->x(),viewport->y()+top_half_begin,viewport->width(),top_half_height);
+            }
+            else
+            {
+                Viewport::value_type bottom_half_height = (viewport->height()-separation)/2;
+                _viewportLeft->setViewport(viewport->x(),viewport->y(),viewport->width(),bottom_half_height);
+            }
+        }
+        break;
+
+        default:
+            _viewportLeft->setViewport(viewport->x(),viewport->y(),viewport->width(),viewport->height());
+            break;
+    }
+}
+
+void SceneView::computeRightEyeViewport(const osg::Viewport *viewport)
+{
+    if(!viewport) return;
+
+    if(!_viewportRight.valid()) _viewportRight = new osg::Viewport;
+
+    switch(_displaySettings->getStereoMode())
+    {
+        case(osg::DisplaySettings::HORIZONTAL_SPLIT):
+        {
+            Viewport::value_type separation = _displaySettings->getSplitStereoHorizontalSeparation();
+
+            if (_displaySettings->getSplitStereoHorizontalEyeMapping()!=osg::DisplaySettings::LEFT_EYE_LEFT_VIEWPORT)
+            {
+                Viewport::value_type left_half_width = (viewport->width()-separation)/2;
+                _viewportRight->setViewport(viewport->x(),viewport->y(),left_half_width,viewport->height());
+            }
+            else
+            {
+                Viewport::value_type right_half_begin = (viewport->width()+separation)/2;
+                Viewport::value_type right_half_width = viewport->width()-right_half_begin;
+                _viewportRight->setViewport(viewport->x()+right_half_begin,viewport->y(),right_half_width,viewport->height());
+            }
+        }
+        break;
+
+        case(osg::DisplaySettings::VERTICAL_SPLIT):
+        {
+            Viewport::value_type separation = _displaySettings->getSplitStereoVerticalSeparation();
+
+            if (_displaySettings->getSplitStereoVerticalEyeMapping()!=osg::DisplaySettings::LEFT_EYE_TOP_VIEWPORT)
+            {
+                Viewport::value_type top_half_begin = (viewport->height()+separation)/2;
+                Viewport::value_type top_half_height = viewport->height()-top_half_begin;
+                _viewportRight->setViewport(viewport->x(),viewport->y()+top_half_begin,viewport->width(),top_half_height);
+            }
+            else
+            {
+                Viewport::value_type bottom_half_height = (viewport->height()-separation)/2;
+                _viewportRight->setViewport(viewport->x(),viewport->y(),viewport->width(),bottom_half_height);
+            }
+        }
+        break;
+
+        default:
+            _viewportRight->setViewport(viewport->x(),viewport->y(),viewport->width(),viewport->height());
+            break;
+    }
+}
+
+void SceneView::setLightingMode(LightingMode mode)
+{
+    if (mode==_lightingMode) return;
+    
+    if (_lightingMode!=NO_SCENEVIEW_LIGHT)
+    {
+        // remove GL_LIGHTING mode
+        _globalStateSet->removeMode(GL_LIGHTING);
+
+        if (_light.valid())
+        {
+            _globalStateSet->removeAssociatedModes(_light.get());
+        }
+
+    }
+
+    _lightingMode = mode;
+
+    if (_lightingMode!=NO_SCENEVIEW_LIGHT)
+    {
+        // add GL_LIGHTING mode
+        _globalStateSet->setMode(GL_LIGHTING, osg::StateAttribute::ON);
+        if (_light.valid()) 
+        {
+            _globalStateSet->setAssociatedModes(_light.get(), osg::StateAttribute::ON);
+        }
+    }
+}
+
 void SceneView::inheritCullSettings(const osg::CullSettings& settings, unsigned int inheritanceMask)
 {
     if (_camera.valid() && _camera->getView()) 
     {
         if (inheritanceMask & osg::CullSettings::LIGHTING_MODE)
         {
+            LightingMode newLightingMode = _lightingMode;
+        
             switch(_camera->getView()->getLightingMode())
             {
-                case(osg::View::NO_LIGHT): setLightingMode(NO_SCENEVIEW_LIGHT); break;
-                case(osg::View::HEADLIGHT): setLightingMode(HEADLIGHT); break;
-                case(osg::View::SKY_LIGHT): setLightingMode(SKY_LIGHT); break;
+                case(osg::View::NO_LIGHT): newLightingMode = NO_SCENEVIEW_LIGHT; break;
+                case(osg::View::HEADLIGHT): newLightingMode = HEADLIGHT; break;
+                case(osg::View::SKY_LIGHT): newLightingMode = SKY_LIGHT; break;
+            }
+            
+            if (newLightingMode != _lightingMode)
+            {
+                setLightingMode(newLightingMode);
             }
         }
                 
@@ -605,10 +742,10 @@ void SceneView::cull()
         osg::notify(osg::INFO) << "Warning: no valid osgUtil::SceneView:: attached, creating a default CullVisitor automatically."<< std::endl;
         _cullVisitor = CullVisitor::create();
     }
-    if (!_rendergraph)
+    if (!_stateGraph)
     {
         osg::notify(osg::INFO) << "Warning: no valid osgUtil::SceneView:: attached, creating a global default StateGraph automatically."<< std::endl;
-        _rendergraph = new StateGraph;
+        _stateGraph = new StateGraph;
     }
     if (!_renderStage)
     {
@@ -623,7 +760,8 @@ void SceneView::cull()
         {
             // set up the left eye.
             _cullVisitor->setTraversalMask(_cullMaskLeft);
-            bool computeNearFar = cullStage(computeLeftEyeProjection(getProjectionMatrix()),computeLeftEyeView(getViewMatrix()),_cullVisitor.get(),_rendergraph.get(),_renderStage.get());
+            computeLeftEyeViewport(getViewport());
+            bool computeNearFar = cullStage(computeLeftEyeProjection(getProjectionMatrix()),computeLeftEyeView(getViewMatrix()),_cullVisitor.get(),_stateGraph.get(),_renderStage.get(),_viewportLeft.get());
 
             if (computeNearFar)
             {
@@ -637,7 +775,8 @@ void SceneView::cull()
         {
             // set up the right eye.
             _cullVisitor->setTraversalMask(_cullMaskRight);
-            bool computeNearFar = cullStage(computeRightEyeProjection(getProjectionMatrix()),computeRightEyeView(getViewMatrix()),_cullVisitor.get(),_rendergraph.get(),_renderStage.get());
+            computeRightEyeViewport(getViewport());
+            bool computeNearFar = cullStage(computeRightEyeProjection(getProjectionMatrix()),computeRightEyeView(getViewMatrix()),_cullVisitor.get(),_stateGraph.get(),_renderStage.get(),_viewportRight.get());
 
             if (computeNearFar)
             {
@@ -651,24 +790,26 @@ void SceneView::cull()
         {
 
             if (!_cullVisitorLeft.valid()) _cullVisitorLeft = dynamic_cast<CullVisitor*>(_cullVisitor->clone());
-            if (!_rendergraphLeft.valid()) _rendergraphLeft = dynamic_cast<StateGraph*>(_rendergraph->cloneType());
+            if (!_stateGraphLeft.valid()) _stateGraphLeft = dynamic_cast<StateGraph*>(_stateGraph->cloneType());
             if (!_renderStageLeft.valid()) _renderStageLeft = dynamic_cast<RenderStage*>(_renderStage->clone(osg::CopyOp::DEEP_COPY_ALL));
 
             if (!_cullVisitorRight.valid()) _cullVisitorRight = dynamic_cast<CullVisitor*>(_cullVisitor->clone());
-            if (!_rendergraphRight.valid()) _rendergraphRight = dynamic_cast<StateGraph*>(_rendergraph->cloneType());
+            if (!_stateGraphRight.valid()) _stateGraphRight = dynamic_cast<StateGraph*>(_stateGraph->cloneType());
             if (!_renderStageRight.valid()) _renderStageRight = dynamic_cast<RenderStage*>(_renderStage->clone(osg::CopyOp::DEEP_COPY_ALL));
             
             _cullVisitorLeft->setDatabaseRequestHandler(_cullVisitor->getDatabaseRequestHandler());
             _cullVisitorLeft->setClampProjectionMatrixCallback(_cullVisitor->getClampProjectionMatrixCallback());
             _cullVisitorLeft->setTraversalMask(_cullMaskLeft);
-            bool computeNearFar = cullStage(computeLeftEyeProjection(getProjectionMatrix()),computeLeftEyeView(getViewMatrix()),_cullVisitorLeft.get(),_rendergraphLeft.get(),_renderStageLeft.get());
+            computeLeftEyeViewport(getViewport());
+            bool computeNearFar = cullStage(computeLeftEyeProjection(getProjectionMatrix()),computeLeftEyeView(getViewMatrix()),_cullVisitorLeft.get(),_stateGraphLeft.get(),_renderStageLeft.get(),_viewportLeft.get());
 
 
             // set up the right eye.
             _cullVisitorRight->setDatabaseRequestHandler(_cullVisitor->getDatabaseRequestHandler());
             _cullVisitorRight->setClampProjectionMatrixCallback(_cullVisitor->getClampProjectionMatrixCallback());
             _cullVisitorRight->setTraversalMask(_cullMaskRight);
-            computeNearFar = cullStage(computeRightEyeProjection(getProjectionMatrix()),computeRightEyeView(getViewMatrix()),_cullVisitorRight.get(),_rendergraphRight.get(),_renderStageRight.get());
+            computeRightEyeViewport(getViewport());
+            computeNearFar = cullStage(computeRightEyeProjection(getProjectionMatrix()),computeRightEyeView(getViewMatrix()),_cullVisitorRight.get(),_stateGraphRight.get(),_renderStageRight.get(),_viewportRight.get());
            
             if (computeNearFar)
             {
@@ -684,7 +825,7 @@ void SceneView::cull()
     {
 
         _cullVisitor->setTraversalMask(_cullMask);
-        bool computeNearFar = cullStage(getProjectionMatrix(),getViewMatrix(),_cullVisitor.get(),_rendergraph.get(),_renderStage.get());
+        bool computeNearFar = cullStage(getProjectionMatrix(),getViewMatrix(),_cullVisitor.get(),_stateGraph.get(),_renderStage.get(),getViewport());
 
         if (computeNearFar)
         {
@@ -697,10 +838,10 @@ void SceneView::cull()
     
 }
 
-bool SceneView::cullStage(const osg::Matrixd& projection,const osg::Matrixd& modelview,osgUtil::CullVisitor* cullVisitor, osgUtil::StateGraph* rendergraph, osgUtil::RenderStage* renderStage)
+bool SceneView::cullStage(const osg::Matrixd& projection,const osg::Matrixd& modelview,osgUtil::CullVisitor* cullVisitor, osgUtil::StateGraph* rendergraph, osgUtil::RenderStage* renderStage, osg::Viewport *viewport)
 {
 
-    if (!_camera || !getViewport()) return false;
+    if (!_camera || !viewport) return false;
 
     osg::ref_ptr<RefMatrix> proj = new osg::RefMatrix(projection);
     osg::ref_ptr<RefMatrix> mv = new osg::RefMatrix(modelview);
@@ -726,7 +867,7 @@ bool SceneView::cullStage(const osg::Matrixd& projection,const osg::Matrixd& mod
              _collectOccludersVisitor->setTraversalNumber(_frameStamp->getFrameNumber());
         }
 
-        _collectOccludersVisitor->pushViewport(getViewport());
+        _collectOccludersVisitor->pushViewport(viewport);
         _collectOccludersVisitor->pushProjectionMatrix(proj.get());
         _collectOccludersVisitor->pushModelViewMatrix(mv.get(),osg::Transform::ABSOLUTE_RF);
 
@@ -779,7 +920,7 @@ bool SceneView::cullStage(const osg::Matrixd& projection,const osg::Matrixd& mod
     // achieves a certain amount of frame cohereancy of memory allocation.
     rendergraph->clean();
 
-    renderStage->setViewport(getViewport());
+    renderStage->setViewport(viewport);
     renderStage->setClearColor(_camera->getClearColor());
     renderStage->setClearDepth(_camera->getClearDepth());
     renderStage->setClearAccum(_camera->getClearAccum());
@@ -808,7 +949,7 @@ bool SceneView::cullStage(const osg::Matrixd& projection,const osg::Matrixd& mod
     if (_localStateSet.valid()) cullVisitor->pushStateSet(_localStateSet.get());
 
 
-    cullVisitor->pushViewport(getViewport());
+    cullVisitor->pushViewport(viewport);
     cullVisitor->pushProjectionMatrix(proj.get());
     cullVisitor->pushModelViewMatrix(mv.get(),osg::Transform::ABSOLUTE_RF);
     
@@ -962,7 +1103,7 @@ void SceneView::draw()
                 if (!leftColorMask)
                 {
                     leftColorMask = new osg::ColorMask();
-                    _renderStageRight->setColorMask(leftColorMask);
+                    _renderStageLeft->setColorMask(leftColorMask);
                 }
                 
                 // red
@@ -979,7 +1120,7 @@ void SceneView::draw()
                 
 
                 // ensure that right eye color planes are active.
-                osg::ColorMask* rightColorMask = _renderStageLeft->getColorMask();
+                osg::ColorMask* rightColorMask = _renderStageRight->getColorMask();
                 if (!rightColorMask)
                 {
                     rightColorMask = new osg::ColorMask();
@@ -993,7 +1134,6 @@ void SceneView::draw()
                 // rightColorMask->setMask(false,false,true,true);
 
                 _localStateSet->setAttribute(rightColorMask);
-                _renderStageRight->setColorMask(rightColorMask);
 
                 // draw right eye.
                 _renderStageRight->draw(_renderInfo,previous);
@@ -1025,46 +1165,27 @@ void SceneView::draw()
                 _renderStageLeft->setColorMask(cmask);
                 _renderStageRight->setColorMask(cmask);
 
+                _localStateSet->setAttribute(_viewportLeft.get());
                 _renderStageLeft->drawPreRenderStages(_renderInfo,previous);
+
+                _localStateSet->setAttribute(_viewportRight.get());
                 _renderStageRight->drawPreRenderStages(_renderInfo,previous);
 
                 double separation = _displaySettings->getSplitStereoHorizontalSeparation();
 
                 double  left_half_width = (getViewport()->width()-separation)/2.0;
-                double right_half_begin = (getViewport()->width()+separation)/2.0;
-                double right_half_width = getViewport()->width()-right_half_begin;
-
-                osg::ref_ptr<osg::Viewport> viewportLeft = new osg::Viewport;
-                viewportLeft->setViewport(getViewport()->x(),getViewport()->y(),left_half_width,getViewport()->height());
-
-                osg::ref_ptr<osg::Viewport> viewportRight = new osg::Viewport;
-                viewportRight->setViewport(getViewport()->x()+right_half_begin,getViewport()->y(),right_half_width,getViewport()->height());
 
                 clearArea(static_cast<int>(getViewport()->x()+left_half_width),
                           static_cast<int>(getViewport()->y()),
                           static_cast<int>(separation),
                           static_cast<int>(getViewport()->height()),_renderStageLeft->getClearColor());
 
-                if (_displaySettings->getSplitStereoHorizontalEyeMapping()==osg::DisplaySettings::LEFT_EYE_LEFT_VIEWPORT)
-                {
-                    _localStateSet->setAttribute(viewportLeft.get());
-                    _renderStageLeft->setViewport(viewportLeft.get());
-                    _renderStageLeft->draw(_renderInfo,previous);
+             
+                _localStateSet->setAttribute(_viewportLeft.get());
+                _renderStageLeft->draw(_renderInfo,previous);
 
-                    _localStateSet->setAttribute(viewportRight.get());
-                    _renderStageRight->setViewport(viewportRight.get());
-                    _renderStageRight->draw(_renderInfo,previous);
-                }
-                else
-                {
-                    _localStateSet->setAttribute(viewportRight.get());
-                    _renderStageLeft->setViewport(viewportRight.get());
-                    _renderStageLeft->draw(_renderInfo,previous);
-
-                    _localStateSet->setAttribute(viewportLeft.get());
-                    _renderStageRight->setViewport(viewportLeft.get());
-                    _renderStageRight->draw(_renderInfo,previous);
-                }
+                _localStateSet->setAttribute(_viewportRight.get());
+                _renderStageRight->draw(_renderInfo,previous);
 
             }
             break;
@@ -1093,20 +1214,16 @@ void SceneView::draw()
 
                 _renderStageLeft->setColorMask(cmask);
                 _renderStageRight->setColorMask(cmask);
+
+                _localStateSet->setAttribute(_viewportLeft.get());
                 _renderStageLeft->drawPreRenderStages(_renderInfo,previous);
+
+                _localStateSet->setAttribute(_viewportRight.get());
                 _renderStageRight->drawPreRenderStages(_renderInfo,previous);
 
                 double separation = _displaySettings->getSplitStereoVerticalSeparation();
 
                 double bottom_half_height = (getViewport()->height()-separation)/2.0;
-                double top_half_begin = (getViewport()->height()+separation)/2.0;
-                double top_half_height = getViewport()->height()-top_half_begin;
-
-                osg::ref_ptr<osg::Viewport> viewportTop = new osg::Viewport;
-                viewportTop->setViewport(getViewport()->x(),getViewport()->y()+top_half_begin,getViewport()->width(),top_half_height);
-
-                osg::ref_ptr<osg::Viewport> viewportBottom = new osg::Viewport;
-                viewportBottom->setViewport(getViewport()->x(),getViewport()->y(),getViewport()->width(),bottom_half_height);
 
                 clearArea(static_cast<int>(getViewport()->x()),
                           static_cast<int>(getViewport()->y()+bottom_half_height),
@@ -1114,26 +1231,11 @@ void SceneView::draw()
                           static_cast<int>(separation),
                           _renderStageLeft->getClearColor());
 
-                if (_displaySettings->getSplitStereoVerticalEyeMapping()==osg::DisplaySettings::LEFT_EYE_TOP_VIEWPORT)
-                {
-                    _localStateSet->setAttribute(viewportTop.get());
-                    _renderStageLeft->setViewport(viewportTop.get());
-                    _renderStageLeft->draw(_renderInfo,previous);
+                _localStateSet->setAttribute(_viewportLeft.get());
+                _renderStageLeft->draw(_renderInfo,previous);
 
-                    _localStateSet->setAttribute(viewportBottom.get());
-                    _renderStageRight->setViewport(viewportBottom.get());
-                    _renderStageRight->draw(_renderInfo,previous);
-                }
-                else
-                {
-                    _localStateSet->setAttribute(viewportBottom.get());
-                    _renderStageLeft->setViewport(viewportBottom.get());
-                    _renderStageLeft->draw(_renderInfo,previous);
-
-                    _localStateSet->setAttribute(viewportTop.get());
-                    _renderStageRight->setViewport(viewportTop.get());
-                    _renderStageRight->draw(_renderInfo,previous);
-                }
+                _localStateSet->setAttribute(_viewportRight.get());
+                _renderStageRight->draw(_renderInfo,previous);
             }
             break;
         case(osg::DisplaySettings::RIGHT_EYE):
@@ -1204,14 +1306,14 @@ void SceneView::draw()
                     glOrtho(getViewport()->x(), getViewport()->width(), getViewport()->y(), getViewport()->height(), -1.0, 1.0);
                     glMatrixMode(GL_MODELVIEW);
                     glLoadIdentity();    
-                    glDisable(GL_LIGHTING);
-                    glDisable(GL_DEPTH_TEST);
+                    getState()->applyMode(GL_LIGHTING,false);
+                    getState()->applyMode(GL_DEPTH_TEST,false);
                     glStencilMask(~0u);
                     glClear(GL_STENCIL_BUFFER_BIT);
                     glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
                     glStencilFunc(GL_ALWAYS, 1, ~0u);
                     glPolygonStipple(patternVertEven);
-                    glEnable(GL_POLYGON_STIPPLE);
+                    getState()->applyMode(GL_POLYGON_STIPPLE,true);
                     glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
                     
                     glRecti(static_cast<GLint>(getViewport()->x()),
@@ -1220,9 +1322,9 @@ void SceneView::draw()
                             static_cast<GLint>(getViewport()->height()) );
                             
                     glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-                    glDisable(GL_POLYGON_STIPPLE);
-                    glEnable(GL_LIGHTING);
-                    glEnable(GL_DEPTH_TEST);
+                    getState()->applyMode(GL_POLYGON_STIPPLE,false);
+                    getState()->applyMode(GL_LIGHTING,true);
+                    getState()->applyMode(GL_DEPTH_TEST,true);
                     
                     _redrawInterlacedStereoStencilMask = false;
                     _interlacedStereoStencilWidth = static_cast<int>(getViewport()->width());
@@ -1281,14 +1383,14 @@ void SceneView::draw()
                     glOrtho(getViewport()->x(), getViewport()->width(), getViewport()->y(), getViewport()->height(), -1.0, 1.0);
                     glMatrixMode(GL_MODELVIEW);
                     glLoadIdentity();
-                    glDisable(GL_LIGHTING);
-                    glDisable(GL_DEPTH_TEST);
+                    getState()->applyMode(GL_LIGHTING,false);
+                    getState()->applyMode(GL_DEPTH_TEST,false);
                     glStencilMask(~0u);
                     glClear(GL_STENCIL_BUFFER_BIT);
                     glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
                     glStencilFunc(GL_ALWAYS, 1, ~0u);
                     glPolygonStipple(patternHorzEven);
-                    glEnable(GL_POLYGON_STIPPLE);
+                    getState()->applyMode(GL_POLYGON_STIPPLE,true);
                     glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
 
                     glRecti(static_cast<GLint>(getViewport()->x()),
@@ -1297,9 +1399,9 @@ void SceneView::draw()
                             static_cast<GLint>(getViewport()->height()) );
                             
                     glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-                    glDisable(GL_POLYGON_STIPPLE);
-                    glEnable(GL_LIGHTING);
-                    glEnable(GL_DEPTH_TEST);
+                    getState()->applyMode(GL_POLYGON_STIPPLE,false);
+                    getState()->applyMode(GL_LIGHTING,true);
+                    getState()->applyMode(GL_DEPTH_TEST,true);
                     
                     _redrawInterlacedStereoStencilMask = false;
                     _interlacedStereoStencilWidth = static_cast<int>(getViewport()->width());
@@ -1451,11 +1553,8 @@ void SceneView::draw()
 
     if (state->getCheckForGLErrors()!=osg::State::NEVER_CHECK_GL_ERRORS)
     {
-        GLenum errorNo = glGetError();
-        if (errorNo!=GL_NO_ERROR)
+        if (state->checkGLErrors("end of SceneView::draw()"))
         {
-            osg::notify(WARN)<<"Warning: detected OpenGL error '"<<gluErrorString(errorNo)<<"'"<< std::endl;
-
             // go into debug mode of OGL error in a fine grained way to help
             // track down OpenGL errors.
             state->setCheckForGLErrors(osg::State::ONCE_PER_ATTRIBUTE);
