@@ -20,6 +20,7 @@
 #include <osgViewer/api/Win32/PixelBufferWin32>
 
 #include <osg/DeleteHandler>
+#include <osg/ApplicationUsage>
 
 #include <vector>
 #include <map>
@@ -31,6 +32,8 @@ using namespace osgViewer;
 namespace osgViewer
 {
 
+static osg::ApplicationUsageProxy GraphicsWindowWin32_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_WIN32_NV_MULTIMON_MULTITHREAD_WORKAROUND on/off","Enable/disable duplicate makeCurrentContext call used as workaround for WinXP/NVidia/MultiView/MulitThread isues (pre 178.13 drivers).");
+
 //
 // Defines from the WGL_ARB_pixel_format specification document
 // See http://www.opengl.org/registry/specs/ARB/wgl_pixel_format.txt
@@ -234,17 +237,17 @@ class Win32WindowingSystem : public osg::GraphicsContext::WindowingSystemInterfa
 
     // Return the resolution of specified screen
     // (0,0) is returned if screen is unknown
-    virtual void getScreenResolution( const osg::GraphicsContext::ScreenIdentifier& si, unsigned int& width, unsigned int& height );
+    virtual void getScreenSettings( const osg::GraphicsContext::ScreenIdentifier& si, osg::GraphicsContext::ScreenSettings & resolution );
 
     // Return the bits per pixel of specified screen
     // (0) is returned if screen is unknown
     virtual void getScreenColorDepth( const osg::GraphicsContext::ScreenIdentifier& si, unsigned int& dmBitsPerPel );
 
     // Set the resolution for given screen
-    virtual bool setScreenResolution( const osg::GraphicsContext::ScreenIdentifier& si, unsigned int width, unsigned int height );
+    virtual bool setScreenSettings( const osg::GraphicsContext::ScreenIdentifier& si, const osg::GraphicsContext::ScreenSettings & resolution );
 
-    // Set the refresh rate for given screen
-    virtual bool setScreenRefreshRate( const osg::GraphicsContext::ScreenIdentifier& si, double refreshRate );
+    // Enumerates available resolutions
+    virtual void enumerateScreenSettings(const osg::GraphicsContext::ScreenIdentifier& screenIdentifier, osg::GraphicsContext::ScreenSettingsList & resolution);
 
     // Return the screen position and width/height.
     // all zeros returned if screen is unknown
@@ -477,7 +480,7 @@ class Win32KeyboardMap
             KeyMap::const_iterator map = _keymap.find(key);
             return map==_keymap.end() ? key : map->second;
         }
-      
+
     protected:
 
         typedef std::map<int, int> KeyMap;
@@ -797,21 +800,40 @@ bool Win32WindowingSystem::getScreenInformation( const osg::GraphicsContext::Scr
     return true;
 }
 
-void Win32WindowingSystem::getScreenResolution( const osg::GraphicsContext::ScreenIdentifier& si, unsigned int& width, unsigned int& height )
+void Win32WindowingSystem::getScreenSettings( const osg::GraphicsContext::ScreenIdentifier& si, osg::GraphicsContext::ScreenSettings & resolution )
 {
     DISPLAY_DEVICE displayDevice;
     DEVMODE        deviceMode;
 
-    if (getScreenInformation(si, displayDevice, deviceMode))
-    {
-        width  = deviceMode.dmPelsWidth;
-        height = deviceMode.dmPelsHeight;
+    if (!getScreenInformation(si, displayDevice, deviceMode))
+        deviceMode.dmFields = 0;        // Set the fields to 0 so that it says 'nothing'.
+
+    // Get resolution
+    if ((deviceMode.dmFields & (DM_PELSWIDTH | DM_PELSHEIGHT)) != 0) {
+        resolution.width  = deviceMode.dmPelsWidth;
+        resolution.height = deviceMode.dmPelsHeight;
+    } else {
+        resolution.width  = 0;
+        resolution.height = 0;
     }
+
+    // Get refersh rate
+    if ((deviceMode.dmFields & DM_DISPLAYFREQUENCY) != 0) {
+        resolution.refreshRate = deviceMode.dmDisplayFrequency;
+        if (resolution.refreshRate == 0 || resolution.refreshRate == 1) {
+            // Windows specific: 0 and 1 represent the hhardware's default refresh rate.
+            // If someone knows how to get this refresh rate (in Hz)...
+            osg::notify(osg::NOTICE) << "Win32WindowingSystem::getScreenSettings() is not fully implemented (cannot retreive the hardware's default refresh rate)."<<std::endl;
+            resolution.refreshRate = 0;
+        }
+    } else
+        resolution.refreshRate = 0;
+
+    // Get bits per pixel for color buffer
+    if ((deviceMode.dmFields & DM_BITSPERPEL) != 0)
+        resolution.colorDepth = deviceMode.dmBitsPerPel;
     else
-    {
-        width  = 0;
-        height = 0;
-    }
+        resolution.colorDepth = 0;
 }
 
 void Win32WindowingSystem::getScreenColorDepth( const osg::GraphicsContext::ScreenIdentifier& si, unsigned int& dmBitsPerPel )
@@ -856,36 +878,65 @@ bool Win32WindowingSystem::changeScreenSettings( const osg::GraphicsContext::Scr
     return false;
 }
 
-bool Win32WindowingSystem::setScreenResolution( const osg::GraphicsContext::ScreenIdentifier& si, unsigned int width, unsigned int height )
+bool Win32WindowingSystem::setScreenSettings( const osg::GraphicsContext::ScreenIdentifier& si, const osg::GraphicsContext::ScreenSettings & resolution )
 {
     DISPLAY_DEVICE displayDevice;
     DEVMODE        deviceMode;
 
     if (!getScreenInformation(si, displayDevice, deviceMode)) return false;
 
-    deviceMode.dmFields     = DM_PELSWIDTH | DM_PELSHEIGHT;
-    deviceMode.dmPelsWidth  = width;
-    deviceMode.dmPelsHeight = height;
+    deviceMode.dmFields = 0;
+    // Set resolution
+    if (resolution.width>0 && resolution.height>0) {
+        deviceMode.dmFields    |= DM_PELSWIDTH | DM_PELSHEIGHT;
+        deviceMode.dmPelsWidth  = static_cast<DWORD>(resolution.width);
+        deviceMode.dmPelsHeight = static_cast<DWORD>(resolution.height);
+    }
+    // Set refersh rate
+    if (resolution.refreshRate>0) {
+        deviceMode.dmFields           |= DM_DISPLAYFREQUENCY;
+        deviceMode.dmDisplayFrequency  = static_cast<DWORD>(resolution.refreshRate);
+    }
+    // Set bits per pixel for color buffer
+    if (resolution.colorDepth>0) {
+        deviceMode.dmFields     |= DM_BITSPERPEL;
+        deviceMode.dmBitsPerPel  = static_cast<DWORD>(resolution.colorDepth);
+    }
     
     return changeScreenSettings(si, displayDevice, deviceMode);
 }
 
-bool Win32WindowingSystem::setScreenRefreshRate( const osg::GraphicsContext::ScreenIdentifier& si, double refreshRate )
-{
-    DISPLAY_DEVICE displayDevice;
-    DEVMODE        deviceMode;
+void Win32WindowingSystem::enumerateScreenSettings(const osg::GraphicsContext::ScreenIdentifier& si, osg::GraphicsContext::ScreenSettingsList & resolutionList) {
+    resolutionList.clear();
 
-    unsigned int width, height;
-    getScreenResolution(si, width, height);
+    if (si.displayNum>0)
+    {
+        osg::notify(osg::WARN) << "Win32WindowingSystem::enumerateScreenSettings() - The screen identifier on the Win32 platform must always use display number 0. Value received was " << si.displayNum << std::endl;
+        return;
+    }
 
-    if (!getScreenInformation(si, displayDevice, deviceMode)) return false;
+    DisplayDevices displayDevices;
+    enumerateDisplayDevices(displayDevices);
 
-    deviceMode.dmFields           = DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFREQUENCY;
-    deviceMode.dmPelsWidth        = width;
-    deviceMode.dmPelsHeight       = height;
-    deviceMode.dmDisplayFrequency = refreshRate;
-    
-    return changeScreenSettings(si, displayDevice, deviceMode);
+    if (si.screenNum>=static_cast<int>(displayDevices.size()))
+    {
+        osg::notify(osg::WARN) << "Win32WindowingSystem::enumerateScreenSettings() - Cannot get information for screen " << si.screenNum << " because it does not exist." << std::endl;
+        return;
+    }
+
+    DISPLAY_DEVICE displayDevice = displayDevices[si.screenNum];
+
+    // Do the enumeration
+    DEVMODE deviceMode;
+    static const unsigned int MAX_RESOLUTIONS = 4046;        // Upper limit to avoid infinite (= very long) loop.
+    for (unsigned int i=0; i<MAX_RESOLUTIONS; ++i)
+    {
+        if (!::EnumDisplaySettings(displayDevice.DeviceName, i, &deviceMode))
+            break;
+        deviceMode.dmSize        = sizeof(deviceMode);
+        deviceMode.dmDriverExtra = 0;
+        resolutionList.push_back(osg::GraphicsContext::ScreenSettings(deviceMode.dmPelsWidth, deviceMode.dmPelsHeight, deviceMode.dmDisplayFrequency, deviceMode.dmBitsPerPel));
+    }
 }
 
 void Win32WindowingSystem::getScreenPosition( const osg::GraphicsContext::ScreenIdentifier& si, int& originX, int& originY, unsigned int& width, unsigned int& height )
@@ -977,7 +1028,10 @@ GraphicsWindowWin32::GraphicsWindowWin32( osg::GraphicsContext::Traits* traits )
   _ownsWindow(true),
   _closeWindow(false),
   _destroyWindow(false),
-  _destroying(false)
+  _destroying(false),
+  _mouseCursor(InheritCursor),
+  _appMouseCursor(LeftArrowCursor),
+  _applyWorkaroundForMultimonitorMultithreadNVidiaWin32Issues( false )
 {
     _traits = traits;
     if (_traits->useCursor) setCursor(LeftArrowCursor);
@@ -1011,7 +1065,7 @@ void GraphicsWindowWin32::init()
 {
     if (_initialized) return;
 
-    getEventQueue()->setCurrentEventState(osgGA::GUIEventAdapter::getAccumulatedEventState().get());
+    // getEventQueue()->setCurrentEventState(osgGA::GUIEventAdapter::getAccumulatedEventState().get());
 
     WindowData *windowData = _traits.get() ? dynamic_cast<WindowData*>(_traits->inheritedWindowData.get()) : 0;
     HWND windowHandle = windowData ? windowData->_hwnd : 0;
@@ -1023,6 +1077,38 @@ void GraphicsWindowWin32::init()
 
     _initialized = _ownsWindow ? createWindow() : setWindow(windowHandle);
     _valid       = _initialized;
+
+    // 2008/10/03
+    // Few days ago NVidia released WHQL certified drivers ver 178.13. 
+    // These drivers (as well as former beta ver 177.92) were free from the bug described below. 
+    // So it looks like its high time to make the workaround inactive by default.
+    // If you happen to still use earlier drivers and have problems consider changing to new ones or 
+    // activate OSG_MULTIMONITOR_MULTITHREAD_WIN32_NVIDIA_WORKAROUND macro def through CMake advanced vars.
+#ifdef OSG_MULTIMONITOR_MULTITHREAD_WIN32_NVIDIA_WORKAROUND
+
+    // 2008/05/12
+    // Workaround for Bugs in NVidia drivers for windows XP / multithreaded / dualview / multicore CPU
+    // affects GeForce 6x00, 7x00, 8x00 boards (others were not tested) driver versions 174.xx - 175.xx
+    // pre 174.xx had other issues so reverting is not an option (statitistics, fbo)
+    // drivers release 175.16 is the latest currently available 
+    // 
+    // When using OpenGL in threaded app ( main thread sets up context / renderer thread draws using it )
+    // first wglMakeCurrent seems to not work right and screw OpenGL context driver data: 
+    // 1: succesive drawing shows a number of artifacts in TriangleStrips and TriangleFans 
+    // 2: weird behaviour of FramBufferObjects (glGenFramebuffer generates already generated ids ...)
+    // Looks like repeating wglMakeCurrent call fixes all these issues
+    // wglMakeCurrent call can impact performance so I try to minimize number of 
+    // wglMakeCurrent calls by checking current HDC and GL context
+    // and repeat wglMakeCurrent only when they change for current thread
+
+    _applyWorkaroundForMultimonitorMultithreadNVidiaWin32Issues = true;
+#endif
+
+    const char* str = getenv("OSG_WIN32_NV_MULTIMON_MULTITHREAD_WORKAROUND");
+    if (str)
+    {
+        _applyWorkaroundForMultimonitorMultithreadNVidiaWin32Issues = (strcmp(str, "on")==0 || strcmp(str, "ON")==0 || strcmp(str, "On")==0 );
+    }
 }
 
 bool GraphicsWindowWin32::createWindow()
@@ -1087,6 +1173,20 @@ bool GraphicsWindowWin32::createWindow()
         return false;
     }
 
+    //
+    // Create the OpenGL rendering context associated with this window
+    //
+
+    _hglrc = ::wglCreateContext(_hdc);
+    if (_hglrc==0)
+    {
+        reportErrorForScreen("GraphicsWindowWin32::createWindow() - Unable to create OpenGL rendering context", _traits->screenNum, ::GetLastError());
+        ::ReleaseDC(_hwnd, _hdc);
+        _hdc  = 0;
+        destroyWindow();
+        return false;
+    }
+
     Win32WindowingSystem::getInterface()->registerWindow(_hwnd, this);
     return true;
 }
@@ -1124,31 +1224,23 @@ bool GraphicsWindowWin32::setWindow( HWND handle )
     // Check if we must set the pixel format of the inherited window
     //
 
-    if (_traits.valid() && _traits->setInheritedWindowPixelFormat)
+    if (!setPixelFormat())
     {
-        if (!setPixelFormat())
-        {
-            reportErrorForScreen("GraphicsWindowWin32::setWindow() - Unable to set the inherited window pixel format", _traits->screenNum, ::GetLastError());
-            _hdc  = 0;
-            _hwnd = 0;
-            return false;
-        }
+        reportErrorForScreen("GraphicsWindowWin32::setWindow() - Unable to set the inherited window pixel format", _traits->screenNum, ::GetLastError());
+        ::ReleaseDC(_hwnd, _hdc);
+        _hdc  = 0;
+        _hwnd = 0;
+        return false;
     }
-    else
-    {
-        //
-        // Create the OpenGL rendering context associated with this window
-        //
 
-        _hglrc = ::wglCreateContext(_hdc);
-        if (_hglrc==0)
-        {
-            reportErrorForScreen("GraphicsWindowWin32::setWindow() - Unable to create OpenGL rendering context", _traits->screenNum, ::GetLastError());
-            ::ReleaseDC(_hwnd, _hdc);
-            _hdc  = 0;
-            _hwnd = 0;
-            return false;
-        }
+    _hglrc = ::wglCreateContext(_hdc);
+    if (_hglrc==0)
+    {
+        reportErrorForScreen("GraphicsWindowWin32::setWindow() - Unable to create OpenGL rendering context", _traits->screenNum, ::GetLastError());
+        ::ReleaseDC(_hwnd, _hdc);
+        _hdc  = 0;
+        _hwnd = 0;
+        return false;
     }
 
     if (!registerWindowProcedure())
@@ -1475,17 +1567,6 @@ bool GraphicsWindowWin32::setPixelFormat()
         return false;
     }
 
-    //
-    // Create the OpenGL rendering context associated with this window
-    //
-
-    _hglrc = ::wglCreateContext(_hdc);
-    if (_hglrc==0)
-    {
-        reportErrorForScreen("GraphicsWindowWin32::setPixelFormat() - Unable to create OpenGL rendering context", _traits->screenNum, ::GetLastError());
-        return false;
-    }
-
     return true;
 }
 
@@ -1657,6 +1738,19 @@ bool GraphicsWindowWin32::makeCurrentImplementation()
         return false;
     }
 
+    if( _applyWorkaroundForMultimonitorMultithreadNVidiaWin32Issues )
+    {
+        if( ::wglGetCurrentDC() != _hdc ||
+            ::wglGetCurrentContext() != _hglrc ) 
+        { 
+            if (!::wglMakeCurrent(_hdc, _hglrc))
+            {
+                reportErrorForScreen("GraphicsWindowWin32::makeCurrentImplementation() - Unable to set current OpenGL rendering context", _traits->screenNum, ::GetLastError());
+                return false;
+            }
+        }
+    }
+
     if (!::wglMakeCurrent(_hdc, _hglrc))
     {
         reportErrorForScreen("GraphicsWindowWin32::makeCurrentImplementation() - Unable to set current OpenGL rendering context", _traits->screenNum, ::GetLastError());
@@ -1689,7 +1783,7 @@ void GraphicsWindowWin32::closeImplementation()
 void GraphicsWindowWin32::swapBuffersImplementation()
 {
     if (!_realized) return;
-    if (!::SwapBuffers(_hdc))
+    if (!::SwapBuffers(_hdc) && ::GetLastError() != 0)
     {
         reportErrorForScreen("GraphicsWindowWin32::swapBuffersImplementation() - Unable to swap display buffers", _traits->screenNum, ::GetLastError());
     }
@@ -1700,7 +1794,7 @@ void GraphicsWindowWin32::checkEvents()
     if (!_realized) return;
 
     MSG msg;
-    while (::PeekMessage(&msg, _hwnd, NULL, NULL, PM_REMOVE))
+    while (::PeekMessage(&msg, _hwnd, 0, 0, PM_REMOVE))
     {
         ::TranslateMessage(&msg);
         ::DispatchMessage(&msg);
@@ -1774,14 +1868,15 @@ void GraphicsWindowWin32::requestWarpPointer( float x, float y )
 #else
     // MIKEC: NEW CODE
     POINT pt;
-    pt.x=x;
-    pt.y=y;
+    pt.x = (LONG)x;
+    pt.y = (LONG)y;
+
     // convert point in client area coordinates to screen coordinates
-    if (!::ClientToScreen(_hwnd,&pt))
+    if (!::ClientToScreen(_hwnd, &pt))
     {
         reportErrorForScreen("GraphicsWindowWin32::requestWarpPointer() - Unable to convert cursor position to screen coordinates", _traits->screenNum, ::GetLastError());
     }
-    if (!::SetCursorPos(pt.x,pt.y))
+    if (!::SetCursorPos(pt.x, pt.y))
     {
         reportErrorForScreen("GraphicsWindowWin32::requestWarpPointer() - Unable to set cursor position", _traits->screenNum, ::GetLastError());
         return;
@@ -1841,12 +1936,28 @@ void GraphicsWindowWin32::useCursor( bool cursorOn )
 
 void GraphicsWindowWin32::setCursor( MouseCursor mouseCursor )
 {
-    _mouseCursor = mouseCursor;
-    HCURSOR newCursor = getOrCreateCursor( mouseCursor);
-    if (newCursor == _currentCursor) return;
+    if (_mouseCursor != mouseCursor)
+    {
+        if (mouseCursor != LeftRightCursor && 
+            mouseCursor != UpDownCursor && 
+            mouseCursor != TopLeftCorner && 
+            mouseCursor != TopRightCorner && 
+            mouseCursor != BottomLeftCorner && 
+            mouseCursor != BottomRightCorner)
+        {
+            _appMouseCursor = mouseCursor;
+        }
 
-    _currentCursor = newCursor;
-    _traits->useCursor = (_currentCursor != NULL);
+        _mouseCursor = mouseCursor;
+        HCURSOR newCursor = getOrCreateCursor( mouseCursor);
+        if (newCursor == _currentCursor) return;
+
+        _currentCursor = newCursor;
+        _traits->useCursor = (_currentCursor != NULL);
+
+        if (_mouseCursor != InheritCursor)
+            ::SetCursor(_currentCursor);
+    }
 }
 
 HCURSOR GraphicsWindowWin32::getOrCreateCursor(MouseCursor mouseCursor)
@@ -1859,7 +1970,7 @@ HCURSOR GraphicsWindowWin32::getOrCreateCursor(MouseCursor mouseCursor)
         _mouseCursorMap[mouseCursor] = NULL;
     break;
     case RightArrowCursor:
-    _mouseCursorMap[mouseCursor] = LoadCursor( NULL, IDC_ARROW);
+        _mouseCursorMap[mouseCursor] = LoadCursor( NULL, IDC_ARROW);
         break;
     case LeftArrowCursor:
         _mouseCursorMap[mouseCursor] = LoadCursor( NULL, IDC_ARROW);
@@ -1918,6 +2029,8 @@ HCURSOR GraphicsWindowWin32::getOrCreateCursor(MouseCursor mouseCursor)
     case BottomLeftCorner:
         _mouseCursorMap[mouseCursor] = LoadCursor( NULL, IDC_SIZENESW );
         break;
+    default:
+        break;
     }
     
     return _mouseCursorMap[mouseCursor];
@@ -2155,6 +2268,7 @@ LRESULT GraphicsWindowWin32::handleNativeWindowingEvent( HWND hwnd, UINT uMsg, W
                 int keySymbol = 0;
                 unsigned int modifierMask = 0;
                 adaptKey(wParam, lParam, keySymbol, modifierMask);
+                _keyMap[keySymbol] = true;
                 //getEventQueue()->getCurrentEventState()->setModKeyMask(modifierMask);
                 getEventQueue()->keyPress(keySymbol, eventTime);
             }
@@ -2169,6 +2283,7 @@ LRESULT GraphicsWindowWin32::handleNativeWindowingEvent( HWND hwnd, UINT uMsg, W
                 int keySymbol = 0;
                 unsigned int modifierMask = 0;
                 adaptKey(wParam, lParam, keySymbol, modifierMask);
+                _keyMap[keySymbol] = false;
                 //getEventQueue()->getCurrentEventState()->setModKeyMask(modifierMask);
                 getEventQueue()->keyRelease(keySymbol, eventTime);
             }
@@ -2189,6 +2304,77 @@ LRESULT GraphicsWindowWin32::handleNativeWindowingEvent( HWND hwnd, UINT uMsg, W
             }
             break;
 
+        ///////////////////
+        case WM_SETFOCUS :
+        ///////////////////
+
+            // Check keys and send a message if the key is pressed when the 
+            // focus comes back to the window.
+            // I don't really like this hard-coded loop, but the key codes
+            // (VK_* constants) seem to go from 0x08 to 0xFE so it should be
+            // ok. See winuser.h for the key codes.
+            for (unsigned int i = 0x08; i < 0xFF; i++)
+            {
+                if ((::GetAsyncKeyState(i) & 0x8000) != 0)
+                    ::SendMessage(hwnd, WM_KEYDOWN, i, 0);
+            }
+            break;
+
+        ///////////////////
+        case WM_KILLFOCUS :
+        ///////////////////
+
+            // Release all keys that were pressed when the window lost focus.
+            for (std::map<int, bool>::iterator key = _keyMap.begin();
+                 key != _keyMap.end(); ++key)
+            {
+                if (key->second)
+                {
+                    getEventQueue()->keyRelease(key->first);
+                    key->second = false;
+                }
+            }
+            break;
+
+        ///////////////////
+        case WM_NCHITTEST :
+        ///////////////////
+            {
+                LONG_PTR result = _windowProcedure==0 ? ::DefWindowProc(hwnd, uMsg, wParam, lParam) :
+                                                        ::CallWindowProc(_windowProcedure, hwnd, uMsg, wParam, lParam);
+
+                switch(result)
+                {
+                case HTLEFT:
+                case HTRIGHT:
+                    setCursor(LeftRightCursor);
+                    break;
+                case HTTOP:
+                case HTBOTTOM:
+                    setCursor(UpDownCursor);
+                    break;
+                case HTTOPLEFT:
+                    setCursor(TopLeftCorner);
+                    break;
+                case HTTOPRIGHT:
+                    setCursor(TopRightCorner);
+                    break;
+                case HTBOTTOMLEFT:
+                    setCursor(BottomLeftCorner);
+                    break;
+                case HTBOTTOMRIGHT:
+                case HTGROWBOX:
+                    setCursor(BottomRightCorner);
+                    break;
+                default:
+                    if (_traits->useCursor && _appMouseCursor != InheritCursor)
+                        setCursor(_appMouseCursor);
+                    break;
+                }
+                return result;
+            }
+            break;
+
         /////////////////
         case WM_CLOSE   :
         /////////////////
@@ -2254,7 +2440,7 @@ struct RegisterWindowingSystemInterfaceProxy
 
 static RegisterWindowingSystemInterfaceProxy createWindowingSystemInterfaceProxy;
 
-}; // namespace OsgViewer
+} // namespace OsgViewer
 
 
 // declare C entry point for static compilation.
@@ -2262,3 +2448,13 @@ extern "C" void graphicswindow_Win32(void)
 {
     osg::GraphicsContext::setWindowingSystemInterface(osgViewer::Win32WindowingSystem::getInterface());
 }
+
+
+void GraphicsWindowWin32::raiseWindow()
+{
+
+    SetWindowPos(_hwnd, HWND_TOPMOST, _traits->x, _traits->y, _traits->width, _traits->height, SWP_NOMOVE|SWP_NOSIZE);
+    SetWindowPos(_hwnd, HWND_NOTOPMOST, _traits->x, _traits->y, _traits->width, _traits->height, SWP_NOMOVE|SWP_NOSIZE);
+
+}
+
