@@ -545,7 +545,7 @@ void GraphicsWindowX11::init()
         return;
     }
 
-    getEventQueue()->setCurrentEventState(osgGA::GUIEventAdapter::getAccumulatedEventState().get());
+    // getEventQueue()->setCurrentEventState(osgGA::GUIEventAdapter::getAccumulatedEventState().get());
 
     WindowData* inheritedWindowData = dynamic_cast<WindowData*>(_traits->inheritedWindowData.get());
     Window windowHandle = inheritedWindowData ? inheritedWindowData->_window : 0;
@@ -655,11 +655,12 @@ bool GraphicsWindowX11::createWindow()
     swatt.event_mask =  0;
     unsigned long mask = CWBackPixel | CWBorderPixel | CWEventMask | CWColormap;
 
-    bool overrideRedirect = false;
-    if (overrideRedirect)
+    if (_traits->overrideRedirect)
     {
         swatt.override_redirect = true;
         mask |= CWOverrideRedirect;
+        
+        osg::notify(osg::INFO)<<"Setting override redirect"<<std::endl;
     }
 
     _window = XCreateWindow( _display, _parent,
@@ -667,8 +668,8 @@ bool GraphicsWindowX11::createWindow()
                              _traits->y,
                              _traits->width, _traits->height, 0,
                              _visualInfo->depth, InputOutput,
-                             _visualInfo->visual, mask, &swatt );
-                             
+                             _visualInfo->visual, mask, &swatt );                          
+
     if (!_window)
     {
         osg::notify(osg::NOTICE)<<"Error: Unable to create Window."<<std::endl;
@@ -704,9 +705,10 @@ bool GraphicsWindowX11::createWindow()
     // now update the window dimensions to account for any size changes made by the window manager,
     XGetWindowAttributes( _display, _window, &watt );
     
-    if (_traits->width != watt.width && _traits->height != watt.height)
+    if (_traits->x != watt.x || _traits->y != watt.y
+        ||_traits->width != watt.width || _traits->height != watt.height)
     {
-        resized( _traits->x, _traits->y, _traits->width, _traits->height );
+        resized( watt.x, watt.y, watt.width, watt.height );
     }
         
     //osg::notify(osg::NOTICE)<<"After sync apply.x = "<<watt.x<<" watt.y="<<watt.y<<" width="<<watt.width<<" height="<<watt.height<<std::endl;
@@ -903,6 +905,8 @@ void GraphicsWindowX11::checkEvents()
     double eventTime = baseTime;
     double resizeTime = eventTime;
     _timeOfLastCheckEvents = getEventQueue()->getTime();
+    
+    // osg::notify(osg::NOTICE)<<"GraphicsWindowX11::checkEvents() : getEventQueue()->getCurrentEventState()->getGraphicsContext()="<<getEventQueue()->getCurrentEventState()->getGraphicsContext()<<std::endl;
 
     int windowX = _traits->x;
     int windowY = _traits->y;
@@ -993,6 +997,7 @@ void GraphicsWindowX11::checkEvents()
 
             case FocusIn :
                 osg::notify(osg::INFO)<<"FocusIn event received"<<std::endl;
+                flushKeyEvents();
                 break;
 
             case UnmapNotify :
@@ -1038,12 +1043,15 @@ void GraphicsWindowX11::checkEvents()
                 getModifierMap(modMap);
                 syncLocks();
 
+                char keyMap[32];
+                XQueryKeymap(_eventDisplay, keyMap);
+
                 // release normal (non-modifier) keys
                 for (unsigned int key = 8; key < 256; key++)
                 {
                     bool isModifier = keyMapGetKey(modMap, key);
                     if (isModifier) continue;
-                    bool isPressed = keyMapGetKey(ev.xkeymap.key_vector, key);
+                    bool isPressed = keyMapGetKey(keyMap, key);
                     if (!isPressed) forceKey(key, eventTime, false);
                 }
 
@@ -1052,7 +1060,7 @@ void GraphicsWindowX11::checkEvents()
                 {
                     bool isModifier = keyMapGetKey(modMap, key);
                     if (!isModifier) continue;
-                    bool isPressed = keyMapGetKey(ev.xkeymap.key_vector, key);
+                    bool isPressed = keyMapGetKey(keyMap, key);
                     forceKey(key, eventTime, isPressed);
                 }
 
@@ -1061,7 +1069,7 @@ void GraphicsWindowX11::checkEvents()
                 {
                     bool isModifier = keyMapGetKey(modMap, key);
                     if (isModifier) continue;
-                    bool isPressed = keyMapGetKey(ev.xkeymap.key_vector, key);
+                    bool isPressed = keyMapGetKey(keyMap, key);
                     if (isPressed) forceKey(key, eventTime, true);
                 }
                 break;
@@ -1385,6 +1393,13 @@ void GraphicsWindowX11::rescanModifierMapping()
     }
 }
 
+void GraphicsWindowX11::flushKeyEvents()
+{
+    XEvent e;
+    while (XCheckMaskEvent(_eventDisplay, KeyPressMask|KeyReleaseMask, &e))
+        continue;
+}
+
 // Returns char[32] keymap with bits for every modifier key set.
 void GraphicsWindowX11::getModifierMap(char* keymap) const
 {
@@ -1472,7 +1487,10 @@ class X11WindowingSystemInterface : public osg::GraphicsContext::WindowingSystem
     // TODO: Investigate whether or not Robert thinks we should store/restore the original
     // resolution in the destructor; I'm not sure the other ones do this, and it may be the
     // responsibility of the user.
-    bool _setScreen(const osg::GraphicsContext::ScreenIdentifier& si, unsigned int width, unsigned height, double rate) {
+    bool _setScreen(const osg::GraphicsContext::ScreenIdentifier& si, unsigned int width, unsigned int height, unsigned int colorDepth, double rate) {
+        if (colorDepth>0)
+            osg::notify(osg::NOTICE) << "X11WindowingSystemInterface::_setScreen() is not fully implemented (missing depth)."<<std::endl;
+
         Display* display = XOpenDisplay(si.displayName().c_str());
         
         if(display)
@@ -1592,8 +1610,11 @@ public:
             osg::notify(osg::INFO) << "X11WindowingSystemInterface, xInitThreads() multi-threaded X support initialized.\n";
         }
 #endif        
+
     }
 
+
+
     ~X11WindowingSystemInterface()
     {
         if (osg::Referenced::getDeleteHandler())
@@ -1639,41 +1660,109 @@ public:
         }
     }
 
-    virtual void getScreenResolution(const osg::GraphicsContext::ScreenIdentifier& si, unsigned int& width, unsigned int& height)
+    bool supportsRandr(Display* display) const
+    {
+#ifdef OSGVIEWER_USE_XRANDR
+        int event_basep;
+        int error_basep;
+        bool supports_randr = XRRQueryExtension( display, &event_basep, &error_basep );
+        if( supports_randr )
+        {
+            int major, minor;
+            XRRQueryVersion( display, &major, &minor );
+            return ( major > 1 || ( major == 1 && minor >= 2 ) );
+        }
+#endif
+        return false;
+    }
+
+    virtual void getScreenSettings(const osg::GraphicsContext::ScreenIdentifier& si, osg::GraphicsContext::ScreenSettings & resolution )
     {
         Display* display = XOpenDisplay(si.displayName().c_str());
         if(display)
         {
-            width = DisplayWidth(display, si.screenNum);
-            height = DisplayHeight(display, si.screenNum);
+            resolution.width = DisplayWidth(display, si.screenNum);
+            resolution.height = DisplayHeight(display, si.screenNum);
+            resolution.colorDepth = DefaultDepth(display, si.screenNum);
+            resolution.refreshRate = 0;            // Missing call. Need a X11 expert.
+            
             XCloseDisplay(display);
         }
         else
         {
             osg::notify(osg::NOTICE) << "Unable to open display \"" << XDisplayName(si.displayName().c_str()) << "\"."<<std::endl;
-            width = 0;
-            height = 0;
+            resolution.width = 0;
+            resolution.height = 0;
+            resolution.colorDepth = 0;
+            resolution.refreshRate = 0;
         }
     }
 
-    virtual bool setScreenResolution(const osg::GraphicsContext::ScreenIdentifier& si, unsigned int width, unsigned int height)
+    virtual bool setScreenSettings(const osg::GraphicsContext::ScreenIdentifier& si, const osg::GraphicsContext::ScreenSettings & resolution )
     {
 #ifdef OSGVIEWER_USE_XRANDR
-        return _setScreen(si, width, height, 0.0f);
+        _setScreen(si, resolution.width, resolution.height, resolution.colorDepth, resolution.refreshRate);
 #else
-        osg::notify(osg::NOTICE) << "You must build osgViewer with Xrandr 1.2 or higher for setScreenResolution support!" << std::endl;
-        return false;
+        osg::notify(osg::NOTICE) << "You must build osgViewer with Xrandr 1.2 or higher for setScreenSettings support!" << std::endl;
 #endif
+        return false;
     }
 
-    virtual bool setScreenRefreshRate(const osg::GraphicsContext::ScreenIdentifier& si, double rate)
+    virtual void enumerateScreenSettings(const osg::GraphicsContext::ScreenIdentifier& si, osg::GraphicsContext::ScreenSettingsList & resolutionList)
     {
+        resolutionList.clear();
+
+        Display* display = XOpenDisplay(si.displayName().c_str());
+        if(display)
+        {
 #ifdef OSGVIEWER_USE_XRANDR
-        return _setScreen(si, 0, 0, rate);
-#else
-        osg::notify(osg::NOTICE) << "You must build osgViewer with Xrandr 1.2 or higher for setScreenRefreshRate support!" << std::endl;
-        return false;
-#endif
+            int defaultDepth = DefaultDepth(display, si.screenNum);
+
+            if (supportsRandr(display))
+            {
+                int nsizes = 0;
+                XRRScreenSize * screenSizes = XRRSizes(display, si.screenNum, &nsizes);
+                if (screenSizes && nsizes>0)
+                {
+                    for(int i=0; i<nsizes; ++i)
+                    {
+                        osg::notify(osg::INFO)<<"Screen size "<<screenSizes[i].width<<" "<<screenSizes[i].height<<" "<<screenSizes[i].mwidth<<" "<<screenSizes[i].mheight<<std::endl;
+
+                        int nrates;
+                        short * rates = XRRRates (display, si.screenNum, i, &nrates);
+                        if (rates && nrates>0)
+                        {
+                            for(int j=0; j<nrates; ++j)
+                            {
+                                osg::notify(osg::INFO)<<"   rates "<<rates[j]<<std::endl;
+                                
+                                resolutionList.push_back(osg::GraphicsContext::ScreenSettings(
+                                    screenSizes[i].width,
+                                    screenSizes[i].height,
+                                    double(rates[j]),
+                                    defaultDepth));
+                            }
+                        }
+                        else
+                        {
+                            resolutionList.push_back(osg::GraphicsContext::ScreenSettings(
+                                screenSizes[i].width,
+                                screenSizes[i].height,
+                                0.0,
+                                defaultDepth));
+                        }
+
+                    }
+                }
+            }
+#endif            
+            XCloseDisplay(display);
+        }
+
+        if (resolutionList.empty())
+        {
+            osg::notify(osg::NOTICE) << "X11WindowingSystemInterface::enumerateScreenSettings() not supported." << std::endl;
+        }
     }
 
     virtual osg::GraphicsContext* createGraphicsContext(osg::GraphicsContext::Traits* traits)
@@ -1730,3 +1819,33 @@ extern "C" void graphicswindow_X11(void)
 {
     osg::GraphicsContext::setWindowingSystemInterface(new X11WindowingSystemInterface);
 }
+
+
+void GraphicsWindowX11::raiseWindow()
+{
+    Display* display = getDisplayToUse();
+    XWindowAttributes winAttrib;
+
+    Window root_return, parent_return, *children;
+    unsigned int nchildren, i=0;
+    XTextProperty windowName;
+    bool xraise = false;
+    
+
+    XQueryTree(display, _parent, &root_return, &parent_return, &children, &nchildren);
+    while (!xraise &&  i<nchildren)
+    {
+    XGetWMName(display,children[i++],&windowName);
+        if ((windowName.nitems != 0) && (strcmp(_traits->windowName.c_str(),(const char *)windowName.value) == 0)) xraise = true;
+    }
+    if (xraise) XRaiseWindow(display,_window);
+    else 
+    {
+    XGetWindowAttributes(display, _window, &winAttrib);
+    XReparentWindow(display, _window, _parent, winAttrib.x, winAttrib.y);
+    }
+    XFree(children);
+
+    XFlush(display); 
+    XSync(display,0);
+}
